function [mse, theta, y, A, b] = fitapprox(app, fun, X, U)
% Fits an approximator to a function

if isfield(app, 'sparse') && app.sparse,
    zeromatrix = @sparse;
else
    zeromatrix = @zeros;
end;
    
Ns = size(X, 2);
A = zeromatrix(Ns, app.N);
b = zeros(Ns, 1);
for i = 1:Ns,
    x = X(:, i); u = U(:, i);
    A(i, :) = app.phi(app, x, u);
    b(i) = fun(x, u);
end;

theta = A \ b;

y = A * theta;
mse = mean((y - b) .^ 2);
