function varargout = fittedqi_exp(mode, cfg)
% Handle experiment in batch run, for this type of algorithm

% TODO implement cleanup

if nargin < 2, cfg = struct; end;



switch mode,
    % provide short ID
    case 'shortid',
        varargout = {'fqi'};
    
    % provide configuration parameters
    case 'params',
        p = struct;
        % main parameters
        p.problem = [];
        p.gamma = [];
        p.U.id = 'U'; p.U.type = 'cell';
        p.regmethod.id = 'regmethod'; p.regmethod.shortid = '';
        % extra-trees parameters
        p.singlereg = [];
        p.trees_ntrees.id = 'trees_ntrees'; p.trees_ntrees.shortid  = 'Ntr'; p.trees_ntrees.label = 'N_{tr}';
        p.trees_k.id = 'trees_k'; p.trees_k.shortid  = 'ktr'; p.trees_k.label = 'k_{tr}';
        p.trees_nmin.id = 'trees_nmin'; p.trees_nmin.shortid  = 'nmin'; p.trees_nmin.label = 'n_{min,tr}';
        % NN parameters
        % TODO nn_sizes, nn_tfs, nn_trainfun, nn_epochs
        % convergence
        p.Xtest.id = 'Xtest'; p.Xtest.type = 'array';       % can be array (cell array of vectors or samples)
        p.Utest.id = 'Utest'; p.Utest.type = 'array';       % similar
        % sample parameters
        p.samplesmethod.path = 'samples.method'; p.samplesmethod.shortid = '';
        p.samplesN.path = 'samples.N'; p.samplesN.shortid = 'N';
        p.samplesgridfun.path = 'samples.gridfun'; p.samplesgridfun.shortid = 'fun';
        p.samplesxgrids.path = 'samples.xgrids'; p.samplesxgrids.type = 'vector'; p.samplesxgrids.shortid = 'xgr';   
            % note we assume we won't explicitly use cell arrays of points, 
            % thus allowing for multiple settings using cell array of vectors
        p.samplesugrids.path = 'samples.ugrids'; p.samplesugrids.type = 'array'; p.samplesugrids.shortid = 'ugr';
        % remaining, "trivial" parameters
        p = addtrivialparams(p, 'maxiter', 'eps', 'datadir', 'verb', 'iterdisp', 'itersave', 'storehistory');
        varargout = {p};
        
    % provide output variables
    case 'outputs',
        o = struct;
        o.score.id = 'score';
        o.score.label = 'score';
        o.score.params = {'gamma', 'X0', 'mc_eps', 'mc_N'};
        
        o.exectime.id = 'exectime';
        o.exectime.label = 'execution time';
        o.exectime.units = 's';
        o.exectime.scale = 'log';

        o.iter.id = 'iter';
        o.iter.label = 'number of iterations';
        
        varargout = {o};
        
    % process options
    case {'run', 'sol'},
        cfg.(mode) = 1;
        fittedqi(cfg);
        
    % compute execution time
    case 'exectime',
        load(cfg.datafile, 'trun');
        varargout = {trun};

    % find # iterations to completion
    case 'iter',
        load(cfg.datafile, 'k');
        varargout = {k-1};

    % evaluate score
    case 'score',
        load(cfg.datafile, 'model', 'approx');
        mccfg.gamma = cfg.gamma;
        mccfg.mc_eps = cfg.mc_eps;
        mccfg.mc_N = cfg.mc_N;
        mccfg.U = approx.U;
        % expand Kd steps to take extra advantage of extratrees vectorization
        mccfg.Kd = 5;       
        % if X0 is string, call problem function to get actual states
        if ischar(cfg.X0), cfg.X0 = feval(model.problem, 'X0', cfg.X0); end;
        score = mc_approx_vectorized(model, approx, [], cfg.X0, mccfg);
        varargout = {score};
        
    otherwise,  % return empty
        varargout = {[]};
end;

