% Script to test DOO and SOO optimization

% Just replace 'doo' with 'soo' to run the other algorithm. Note due to
% some tricks which were not explained in the lecture, SOO will not run
% unchanged on the quadratic functions

% The anonymous function 'dmax' is only used by SOO, whereas 'diam' is only used by DOO

% Variables are denoted 'u' instead of x like in the lecture
% maxu gives the range of X = [-maxu, maxu]
% n is the node expansions budget

% Each section separated by %% is a cell. Place the cursor in a cell and
% hit CTRL+ENTER to run the cell

%% 1D quadratic fcn
cfg = struct;
cfg.fun = @(u, ustar, delta) delta-(u-ustar)^2;
cfg.funargs = {2, 40};
cfg.maxu = 5;
% the obj function is Lipschitz w.r.t. |x-x'|^2
% change the exponent to something smaller to have a mismatched norm, it will take more expansions
% a larger exponent does NOT satisfy the Lipschitz condition!!
cfg.diam = @(int) abs((int(2)-int(1))/2)^1;
cfg.method = 'doo';
cfg.n = 25;
cfg.dmax = @(tau) sqrt(tau);
cfg.verb = 5;
cfg.viewpause = 0;

[ustar, info, data] = doosoo(cfg);

%% 2D quadratic fcn
cfg = struct;
cfg.fun = @(u, ustar, delta) delta-norm(u-ustar, inf)^2;
cfg.funargs = {[0.33 0.33]', 4};
cfg.maxu = [1 2]';
cfg.diam = @(int) norm((int(2, :)-int(1, :)) / 2, inf)^2;
cfg.method = 'doo';
cfg.n = 25;
cfg.verb = 4;

[ustar, info, data] = doosoo(cfg);

%% 2D Rosenbrock banana
cfg = struct;
cfg.fun = @(x) 500-100*(x(2)-x(1)^2)^2-(1-x(1))^2;
cfg.funargs = {};
cfg.maxu = [1.9; 2.23];
cfg.diam = @(int) 200*norm((int(2, :)-int(1, :)) / 2, 2)^2;
cfg.method = 'soo';
cfg.dmax = @(tau) sqrt(tau);
cfg.n = 100;
cfg.verb = 5;

[ustar, info, data] = doosoo(cfg);
