function [idx,D]=knnsearchx(Q, R, K, S)
% KNNSEARCH   Linear k-nearest neighbor (KNN) search
% IDX = knnsearch(Q,R,K,S) searches the reference data set R (n x d array
% representing n points in a d-dimensional space) to find the k-nearest
% neighbors of each query point represented by eahc row of Q (m x d array).
% The results are stored in the (m x K) index array, IDX. 
%
% The distance metric is diagonally scaled by S

% By Yi Cao at Cranfield University on 25 March 2008
% Modified by LB for speed and customizable distance metric

[N,M] = size(Q);
L=size(R,1);
idx = zeros(N,K);
D = idx;

if K==1
    % Loop for each query point
    for k=1:N
        d=zeros(L,1);
        for t=1:M
            d=d+((R(:,t)-Q(k,t)).*S(t)).^2;
        end
%         if fident
%             d(k)=inf;
%         end
        [D(k),idx(k)]=min(d);
    end
else
    for k=1:N
        d=zeros(L,1);
        for t=1:M
            d=d+((R(:,t)-Q(k,t)).*S(t)).^2;
        end
%         if fident
%             d(k)=inf;
%         end
        [s,t]=sort(d);
        idx(k,:)=t(1:K);
        D(k,:)=s(1:K);
    end
end
if nargout>1
    D=sqrt(D);
end

