function myTreeplot(p,labels)
%TREEPLOT Plot picture of tree.
%   TREEPLOT(p) plots a picture of a tree given a row vector of
%   parent pointers, with p(i) == 0 for a root and labels on each node.
%
%   Example:
%      myTreeplot([2 4 2 0 6 4 6],{'i' 'like' 'labels' 'on' 'pretty' 'trees' '.'})
%   returns a binary tree with labels.
%
%   Copyright 1984-2004 The MathWorks, Inc.
%   $Revision: 5.12.4.2 $  $Date: 2004/06/25 18:52:28 $
%   Modified by Richard @ Socher . org to display text labels

[x,y,h]=treelayout(p);
f = find(p~=0);
pp = p(f);
X = [x(f); x(pp); repmat(NaN,size(f))];
Y = [y(f); y(pp); repmat(NaN,size(f))];
X = X(:);
Y = Y(:);

n = length(p);
if n < 500,
  plot (x, y, 'ro', X, Y, 'r-');
else
  plot (X, Y, 'r-');
end;
xlabel(['height = ' int2str(h)]);
axis([0 1 0 1]);

for l=1:length(labels)
  text(x(l),y(l),labels{l},'Interpreter','none')
end
