function [xplus, rplus, terminal, G] = mainvpend_mg(m, x, u, mode, iagent, G, xplusothers)
%  Discrete-time dynamics of the Markov game for decoupled, homogeneous inverted pendulums.
%  [XPLUS, RPLUS, TERMINAL, G] = mainvpend_mg(M, X, U, MODE, IAGENT, G)
%  Parameters: see malinear_mg

if mode(1) == 'r',  nagents = m.nagents; G = m.G;
else                nagents = 1 + sum(G);
end;
if nargin < 7,      xplusothers = []; end;

x = reshape(x, m.p1, nagents); u = reshape(u, m.q1, nagents);
xplusothers = reshape(xplusothers, m.p1, []); % may be empty

% compute transition
xplus = nan(size(x));
for i = 1:nagents,
    % next state gets picked up from predictions if:
    % we are in prediction mode, and not handling the current agent, and a prediction is available
    if mode(1) == 'p' && i > 1 && ~isempty(xplusothers) && ~any(isnan(xplusothers(:, i-1))),
        xplus(:, i) = xplusothers(:, i-1);
    else    % otherwise, we must simulate
        xplus(:, i) = invpend_discretestep(m, x(:, i), u(:, i));
    end;
end;

% dummy rewards 
rplus = NaN(1, nagents);

xplus = xplus(:);
terminal = 0;

end         % MAIN FUNCTION END =================

