%% OP experiments for flocking
function malinear_opflocking_1d(mode, expids, custombcfg, customacfg)

%% General config
% clear all; close all;

% model and representative initial states
mcfg = struct;
mcfg.type = 'doubleint'; mcfg.nagents = 6; mcfg.dim = 1; mcfg.Ts = 1;
mcfg.range = 5; mcfg.flock_dist = 5; mcfg.flock_maxdvel = 5;

m = malinear_problem('model', mcfg);
p0 = 0:0.4*mcfg.range:0.4*mcfg.range*(mcfg.nagents-1);
[Grob, rhorob, lambda2rob] = malinear_problem('minrobgraph', m, [p0; zeros(1, mcfg.nagents)]);

vrob = lambda2rob * rhorob / sqrt(2) / sqrt(m.nagents);
% case 1): condition satisfied
x0{1} = [p0; vrob * [-ones(1, m.nagents/2) ones(1, m.nagents/2)]];
% case 2): condition violated but with intermediate value
    % note this case requires more than 100 s to converge
    % it was changed on 11 July 2012 since the original 1.25 was too large for the algorithm with
    % gain 1/n (the previous gain was 1/max degrees + 1 but required centralized knowledge)
x0{2} = [p0; 1.2 * vrob * [-ones(1, m.nagents/2) ones(1, m.nagents/2)]]; 
% case 3): condition violated with large value
x0{3} = [p0; 2.5 * vrob * [-ones(1, m.nagents/2) ones(1, m.nagents/2)]];
        

% batch experiment
BCFG = struct;
BCFG.alg = 'magenmpc';
BCFG.verb = 5;
BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'problem', 'planparamsn', 'planparamsmaxpredlength', 'planparamsU'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% algorithms
ACFG = struct;
ACFG.problem = 'malinear_problem';
ACFG.model_params = {mcfg};
ACFG.online = 1;
ACFG.comm = 'prev';
ACFG.gamma = 0.98;
ACFG.planalg = 'maopd_predictneighbors';
ACFG.planparams.U = repmat({symloggrid(7, .3)}, 1, mcfg.nagents);
    % earlier experiments ran with   -0.3000   -0.1500   -0.0500         0    0.0500    0.1500 0.3000
ACFG.planparams.rob = 0;
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 80;
% visualization
ACFG.verb = 3;
ACFG.datadir = 'd:\work\m\approxrl\masystems\malinear\malinear_opflocking';
ACFG.steps = [];
ACFG.reset = x0{3}(:);

cd(ACFG.datadir); 


%% Local execution function 
% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            bcfg = copyfields(custombcfg, bcfg); 
            acfg = copyfields(customacfg, acfg);
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            elseif any(strcmp(mode, {'disagr', 'disconn', 'exectime', 'depth'})), 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

    function execsimpleexp(expid, algo, acfg)
        if any(strcmp(expid, expids)),  % if experiment is requested
            acfg = copyfields(customacfg, acfg);
            acfg.(mode) = 1;
            algo(acfg);
        end;
    end

%% Basic experiments: classical control
acfg = ACFG;
acfg.idinfix = 'classical';
acfg.planalg = 'masimplecontrol';
acfg.planparams.type = 'flock';
% 12 July 2012: gain 1/n < 1/max_degree to ensure convergence 
% (the previous gain was the default 1/max degrees + 1 but this required centralized knowledge)
acfg.planparams.alpha = 1/mcfg.nagents; 
% cfg.visualize = 2;
acfg.viscfg.pause = .01;
acfg.datadir = pwd;
% acfg.maxsteps = 150;
acfg.verb = 0;

acfg.reset = x0{1}(:); acfg.datafile = 'classical_malin_1d_vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
acfg.reset = x0{2}(:); acfg.datafile = 'classical_malin_1d_15vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
acfg.reset = x0{3}(:); acfg.datafile = 'classical_malin_1d_25vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);

%% Sequential communication: basic experiments: OP control

acfg = ACFG;
acfg.idinfix = 'opseq'; 
acfg.planparams.n = 200;
acfg.comm = 'seq';

acfg.reset = x0{1}(:); acfg.datafile = 'magenmpc_seq_malin_1d_vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
acfg.reset = x0{2}(:); acfg.datafile = 'magenmpc_seq_malin_1d_15vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
acfg.reset = x0{3}(:); acfg.datafile = 'magenmpc_seq_malin_1d_25vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);


%% Sequential communication: influence of n // F6Ve

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = '1dbudgetseq';
% bcfg.legendprefix = 'Optimistic';
acfg = ACFG;
acfg.online = 1;
acfg.comm = 'seq';
acfg.planparams.n = [15 25 50 75 100 200 300 400 500];
acfg.planparams.maxpredlength = Inf;

execbatchexp(bcfg.idinfix);


%% Sequential communication: influence of action space discretization // F6Ve

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = '1ddiscseq';
% bcfg.legendprefix = 'Optimistic';
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamsmaxpredlength', 'planparamsU'};
bcfg.in = 'planparamsU';

acfg = ACFG;
acfg.online = 1;
acfg.comm = 'seq';
acfg.planparams.U = {...
    repmat({symloggrid(3, .3)}, 1, mcfg.nagents), ...
    repmat({symloggrid(5, .3)}, 1, mcfg.nagents), ...
    repmat({symloggrid(7, .3)}, 1, mcfg.nagents), ...
    repmat({symloggrid(9, .3)}, 1, mcfg.nagents), ...
    repmat({symloggrid(11, .3)}, 1, mcfg.nagents), ...
    };
acfg.planparams.n = 500;
acfg.planparams.maxpredlength = Inf;

execbatchexp(bcfg.idinfix);

%% Sequential communication: influence of length of communicated predictions // F6Ve

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = '1dpredlenseq';
bcfg.in = 'planparamsmaxpredlength';

acfg = ACFG;
acfg.online = 1;
acfg.comm = 'seq';
acfg.planparams.n = 200;
acfg.planparams.maxpredlength = [0 1 2 3 4 5 Inf];

execbatchexp(bcfg.idinfix);




%% old, previous-plan communcation experiments:

% %% Previous plans communicated: basic experiments: OP control
% 
% acfg = ACFG;
% acfg.idinfix = 'op'; 
% acfg.planparams.n = 200;
% 
% acfg.reset = x0{1}(:); acfg.datafile = 'magenmpc_malin_1d_vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
% acfg.reset = x0{2}(:); acfg.datafile = 'magenmpc_malin_1d_15vrob'; execsimpleexp(acfg.idinfix, @magenmpc, acfg);
% acfg.reset = x0{3}(:); acfg.datafile = 'magenmpc_malin_1d_25vrob'; execsimpleexp(acfg.idinfix,
% @magenmpc, acfg);

% Previous plans communicated: influence of n for a reasonable fixed action space // F6Ve
% 
% bcfg = BCFG;
% bcfg.mode = 'run';
% bcfg.idinfix = '1dbudget';
% % bcfg.legendprefix = 'Optimistic';
% acfg = ACFG;
% acfg.online = 1;
% acfg.planparams.n = [5 10 15 25 50 75 100 200 300 400 500];
% acfg.planparams.maxpredlength = Inf;
% 
% execbatchexp(bcfg.idinfix);
% 
% 
% %% Previous plans communicated: influence of action space discretization // F6Ve
% 
% bcfg = BCFG;
% bcfg.mode = 'run';
% bcfg.idinfix = '1ddisc';
% % bcfg.legendprefix = 'Optimistic';
% bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamsmaxpredlength', 'planparamsU'};
% bcfg.in = 'planparamsU';
% 
% acfg = ACFG;
% acfg.online = 1;
% acfg.planparams.U = {...
%     repmat({symloggrid(3, .3)}, 1, mcfg.nagents), ...
%     repmat({symloggrid(5, .3)}, 1, mcfg.nagents), ...
%     repmat({symloggrid(7, .3)}, 1, mcfg.nagents), ...
%     repmat({symloggrid(9, .3)}, 1, mcfg.nagents), ...
%     repmat({symloggrid(11, .3)}, 1, mcfg.nagents), ...
%     };
% acfg.planparams.n = 500;
% acfg.planparams.maxpredlength = Inf;
% 
% execbatchexp(bcfg.idinfix);
% 
% %% Previous plans communicated: influence of length of communicated predictions // F6Ve
% 
% bcfg = BCFG;
% bcfg.mode = 'run';
% bcfg.idinfix = '1dpredlen';
% bcfg.in = 'planparamsmaxpredlength';
% 
% acfg = ACFG;
% acfg.online = 1;
% acfg.planparams.n = 200;
% acfg.planparams.maxpredlength = [0 1 2 3 4 5 Inf];
% 
% execbatchexp(bcfg.idinfix);

end
