% Test discrete-time flocking with or without minimal robust subgraph condition satisfied
clc; clear all; close all;

%% Small config: 4 agents

mcfg = struct;
mcfg.nagents = 4;
mcfg.range = 5;
mcfg.beta = .5;
mcfg.Ts = 0.5;
mcfg.agree = 3:4;
dvmax = 5; mcfg.Q = eye(length(mcfg.agree)) ./ dvmax^2;

m = manonhol_problem('model', mcfg);

gridstep = 1;
 % setup grid structure in 2D
gr = zeros(2, 2, 2); 
gr(1, :, :) = [-1 -1; 1 1];
gr(2, :, :) = [1 -1; 1 -1];

% initial position (always the same)
p0 = reshape(gr .* gridstep, 2, mcfg.nagents);
x0 = [p0; ones(1, mcfg.nagents); 3*pi/4 pi/4 -3*pi/4 -pi/4];

%% Large config: 9 agents

mcfg = struct;
mcfg.nagents = 9;
mcfg.range = 5;
mcfg.beta = .5;
mcfg.Ts = 0.25;
mcfg.agree = 3:4;
dvmax = 5; mcfg.Q = eye(length(mcfg.agree)) ./ dvmax^2;

gridstep = 3;
 % setup grid structure in 2D
gr = zeros(2, 3, 3); 
gr(1, :, :) = [-1 0 1; -1 0 1; -1 0 1];
gr(2, :, :) = [1 1 1; 0 0 0; -1 -1 -1];
% initial position (always the same)
p0 = reshape(gr .* gridstep, 2, mcfg.nagents);
x0 = [p0; 
    1.5 .*[1 1 1 1 0 1 1 1 1];
    pi .* [3/4 1 -3/4 1/2 0 -1/2 1/4 0 -1/4]];

% x0 = x0(:, randperm(mcfg.nagents));

%% Optimistic planning with sequential communication
cfg = struct;
cfg.problem = 'manonhol_problem';
cfg.model_params = {mcfg};
cfg.comm = 'seq';
cfg.planalg = 'maopd_predictneighbors';
Ugrid = flat({[-0.5 0 0.5], [-pi/3 0 pi/3]});
cfg.planparams.U = repmat({Ugrid}, 1, mcfg.nagents);
cfg.planparams.n = 200;
cfg.planparams.rob = 0;
cfg.planparams.xpredtype = 'flatcomm';
% cfg.planparams.maxpredlength = 1;
cfg.planparams.verb = 2;
cfg.visualize = 2;
cfg.viscfg.pause = .01;
cfg.datadir = pwd;
cfg.maxsteps = 40;
cfg.verb = 3;
cfg.saveplandata = 0;

% run
cfg.reset = x0; cfg.run = 1; cfg.datafile = 'magenmpc_op_nonholon'; magenmpc(cfg);


