function varargout = manonhol_visualize(cfg)
% Visualization function for decoupled linear agents living in Euclidean space

% TODO visualizing from data file: not tested
% output to sequence of figures: not tested

% default arguments
if nargin < 1, cfg = ''; end;

% where from to load the data, one of:
%       'caller'    - if the variables should be taken from the calling function
%       filename    - if data should be loaded from a file with name <filename>
% CFG.problem = 'cleanrob_problem';
CFG.datasource = 'caller';
CFG.gview = [];           % if nonempty: update existing; if empty: create new; graphical view
% CFG.datafile   = '';    % data source can be given as a data file here as well
% update/create view with:
CFG.mampcstep = 0;
% CFG.mampctrial = 0;
CFG.finalize = 0;         % special option to finalize the output (namely, movie)
% which qiter/piter iteration/trial to plot
CFG.trial = [];
CFG.k = [];
CFG.ktotal = [];        % total #of steps from start of experiment

CFG = setfigprop(CFG, 'addfields');  % add figure settings fields to CFG
CFG.figsize = [800 700];
% output configuration
CFG.pause = .1;
CFG.plottarget = 'screen';      % 'latex', 'beamer', 'screen', 'movie'
CFG.snapshottarget = 'screen';  % 'latex', 'beamer', 'screen'
CFG.savedir = ''; 
CFG.savefig = 'nonholvisualization';% filename for saving figure
% if output is "movie":
CFG.compression = 'Cinepak';    % quality with Cinepak is pretty crappy...
CFG.quality = 100;
CFG.fps = 1;
CFG.snapshot = [];              % still snapshot config
% snapshot has the form [iter]

% process config
cfg = parseconfig(cfg, CFG);

% needed vars
vars = {'model', 'cfg'};
if cfg.mampcstep,
    vars = {vars{:}, 'X', 'GRAPHS'}; 
end;

% retrieve needed variables from the data source
cfg1 = cfg;
switch cfg.datasource,
    case 'caller',              % load needed vars from caller space
        for i=1:length(vars), 
            eval(sprintf('%s=evalin(''caller'', ''%s'');', vars{i}, vars{i})); 
        end;
    otherwise,
%         load(cfg.datasource, vars{:});
end;
expcfg = cfg; cfg = cfg1;

% set up style...
sty = struct;
sty.cm = jet(64);
% sty.hcm = gray(128); sty.hcm = sty.hcm(33:end, :);    % w/o strong blacks and whites
sty.title = {'HorizontalAlignment', 'left', 'FontSize', 12, 'FontWeight', 'bold', 'Margin', 1, 'LineStyle', 'none'};
sty.agentcolors = {'b', 'r', 'g', 'k', 'm', 'y', 'c', [.5 .5 .5], [1 .5 .5]};
sty.agentsize = .1;
sty.connections = {'Color', 'k', 'LineStyle', '-', 'LineWidth', 1};
sty.velocity = {'LineStyle', '-', 'LineWidth', 2};
sty.range = {'LineStyle', '--', 'LineWidth', 1};
sty.return = {'Color', 'g', 'LineWidth', 1.5};

% ...and labels
labels = struct;
labels.pos = {'p'}; labels.vel = {'v'}; labels.u = {'u'};  
labels.trial = 'trial'; labels.return = 'R(x_0)'; labels.time = 'k*T_s [s]';

% ===================================
% ==== Prepare data for plotting ====

% flags for what we are going to show
show = struct;
show.world = [];
show.x = [];
show.X = [];
show.return = [];
show.nothing = 0;

if cfg.mampcstep,
    show.world = 1;
    show.x = X(:, cfg.k+1, cfg.trial);
    show.G = GRAPHS(:, :, cfg.k+1, cfg.trial);
    show.X = X(:, 1:cfg.k+1, cfg.trial);
    show.taxis = (0:cfg.k) * model.Ts;
    algname = 'Online planning';
    gtitle = sprintf('%s, trial %d, k=%d', algname, cfg.trial, cfg.ktotal);
    show.worldsubplot = {2, 2, [1 2]};
    show.possubplot = {2, 2, 3}; 
    show.velsubplot = {2, 2, 4};
else
    show.nothing = 1;       % signal nothing is in fact being shown
end;

% create or retrieve the view
if isempty(cfg.gview);  
    create = 1; gview = struct; gview.figh = figure; setfigprop(cfg);
    colormap(sty.cm); 
    gview.title = annotation('textbox', [.05 .93 .5 .05], 'String', gtitle, sty.title{:}) ;
    % open movie if needed
    if strcmp(cfg.plottarget, 'movie'),
        avifilename = [cfg.savedir cfg.savefig '.avi'];
        if exist(avifilename, 'file'), delete(avifilename); end;
        gview.aviobj = avifile(avifilename, 'compression', cfg.compression, 'quality', cfg.quality, 'fps', cfg.fps);
    end;
else
    create = 0; gview = cfg.gview; figure(gview.figh);
end;

% ===================================
% ==== Create world if needed =======
if ~isempty(show.world) && create,
    figure(gview.figh);
    subplot(show.worldsubplot{:});
    for i = 1:model.nagents,
        % draw bullet representing current state, and velocity line
        gview.agents(i) = rectangle('Position', [-.5 -.5 1 1] * sty.agentsize, 'Curvature', [1,1]);
        set(gview.agents(i), 'EdgeColor', 'none', 'FaceColor', sty.agentcolors{i});
        gview.velocities(i) = line([0 0], [0 0]);
        set(gview.velocities(i), 'Color', sty.agentcolors{i}, sty.velocity{:});
        gview.ranges(i) = rectangle('Position', [-1 -1 2 2] * model.range, 'Curvature', [1,1]);
        set(gview.ranges(i), 'EdgeColor', sty.agentcolors{i}, 'FaceColor', 'none', sty.range{:});
    end;
    gview.connections = [];
    axis equal %off manual
    drawnow;
end;

% ===================================================
% ==== Perform the update of the graphical view =====

if ~isempty(show.x),
    subplot(show.worldsubplot{:});
    x = reshape(show.x, model.p(1), model.nagents);
    pos = x(1:2, :); 
    v = x(3, :); theta = x(4, :);
    vel = [v .* cos(theta); v .* sin(theta)];
    % show agents at updated (next) positions, update velocities
    % at the same time make the connection lines
    connx = []; conny = [];
    for i = 1:model.nagents,
        rectcoords = get(gview.agents(i), 'Position');
        set(gview.agents(i), 'Position', [pos(:, i)'-sty.agentsize/2 rectcoords(3:4)]);
        set(gview.ranges(i), 'Position', [-1 -1 2 2] * model.range + [pos(:, i)' 0 0]);
        set(gview.velocities(i), 'XData', [pos(1, i) pos(1, i) + vel(1, i)], ...
            'YData', [pos(2, i) pos(2, i) + vel(2, i)]);
        % note we only consider superdiagonal adjacencies to avoid repeated lines
        for j = i+1:model.nagents,
            if show.G(i, j),
                connx = [connx [pos(1, i); pos(1, j)]];
                conny = [conny [pos(2, i); pos(2, j)]];
            end;
        end;
    end;
    % note connx conny might also be empty!
    if ~isempty(gview.connections), delete(gview.connections); end;
    gview.connections = line(connx, conny, sty.connections{:});
end;

% note we only show the first axis
% -- this is just thrown together, test if using
if ~isempty(show.X),
    X = reshape(show.X, model.p(1), model.nagents, []);
    pos = X(1:2, :, :); 
    v = X(3, :, :); theta = X(4, :, :);
    subplot(show.possubplot{:}); cla; hold on;
    for i = 1:model.nagents,
        plot(show.taxis, squeeze(pos(1, i, :)), 'Color', sty.agentcolors{i}); 
    end;
    xlabel(labels.time); ylabel(labels.pos);
    subplot(show.velsubplot{:}); cla; hold on;
    for i = 1:model.nagents,
        plot(show.taxis, squeeze(v(1, i, :)), 'Color', sty.agentcolors{i});
        plot(show.taxis, squeeze(theta(1, i, :)), '--', 'Color', sty.agentcolors{i});
    end;
    xlabel(labels.time); ylabel(labels.vel);
end;

% change the title if needed
if ~show.nothing,
    set(gview.title, 'String', gtitle);
end;

% ==========================================
% ==== Save graphical output (if any) ======

if ~show.nothing,
    switch cfg.plottarget,
        case 'screen',
            drawnow;
            if isscalar(cfg.pause), 
                if cfg.pause >= 0, pause(cfg.pause),
                else pause;
                end;
            else
                % assuming cell with structure 
                % {condition pause; condition pause; ...}; should include at
                % least a default where the condition evaluates true
                for i = 1:size(cfg.pause, 1),
                    if eval(cfg.pause{i, 1}),
                        if cfg.pause{i, 2} >= 0, pause(cfg.pause{i, 2}),
                        else pause;
                        end;
                        break;
                    end;
                    % otherwise try the next condition
                end;
            end;
        case 'movie',
            % add the figure as a frame to the movie
            drawnow;
            gview.aviobj = addframe(gview.aviobj, gview.figh);
            % determine if we need to take a snapshot at this stage
            if ~isempty(cfg.snapshot),
                if length(cfg.snapshot) == 2,
                    takesnapshot = (cfg.mampcstep) ...
                        && (cfg.trial == cfg.snapshot(1)) && (cfg.k == cfg.snapshot(2));
                else 
                    takesnapshot = 0;
                end;
            else
                takesnapshot = 0;
            end;
            if takesnapshot,
                saveplot(gview.figh, [cfg.savedir cfg.savefig], cfg.snapshottarget);
            end;
    end;
end;

% Finalize movie if requested to do so
if cfg.finalize,
    if strcmp(cfg.plottarget, 'movie'),
        gview.aviobj = close(gview.aviobj);
    end;
end;

varargout = {gview.figh, gview};

end
% END =================================================

