function lspiexpcleanup_crtdir(level)
% adhoc function to cleanup LSPI files to save disk space. uses a copy paste of the lspiexp
% cleanup subfunction

if nargin < 1, level = 1.5; end;  % medium cleanup

files = dir;
for i = 1:length(files),
    extpos = strfind(files(i).name, '.mat');
    if extpos,
        df = files(i).name(1:extpos-1);
        lspiexp_cleanup(df, level); 
    end;
end;

end

% --------------------
% Subfunction to cleanup an LSPI datafile, thus optimizing on the HDD space taken
function lspiexp_cleanup(cleanupdf, level)
% Levels:
%   0.5 	soft cleanup, will compute conditioning numbers for A if they were not computed
%       while running the algorithm. All the new data files should have condition numbers saved,
%       so that this mode should no longer be necessary
%   1       normal cleanup, history of A, b, the values of A, b, and Ainv
%   1.5     medium cleanup, cleans up PHI and HPHI (precomputed BF values)
%       These take a lot of space and not so much time to recompute.
%   2       hard cleanup, cleans up PHI and HPHI, histories of parameter vectors. This means
%       the evolution of the solution over iterations can no longer be examined.
%       Should only be used as a last resort, as parameter vectors don't really take up that
%       much space

if ~exist([cleanupdf '.mat'], 'file'), return; end;
% make sure the same cleanupdf is kept
cdf2 = cleanupdf; load(cleanupdf); cleanupdf = cdf2; clear cdf2;
if exist('cleanedup', 'var') && (cleanedup >= level), return; end;  % nothing to do 

disp(['Level-' num2str(level) ' cleanup of [' cleanupdf ']']);


% remove thetastar in any case, it is no longer used
clear thetastar

K = find(~isnan(deltah), 1, 'last') - 1;    % may be needed by various cleaning levels

% soft cleanup: compute conditioning numbers for A matrices (unless already computed) and store them
if level == 0.5,
    if ~exist('condA', 'var'),
        condA = zeros(1, K);
        for k=1:K,
            if issparse(Ah{k}),     condA(k) = condest(Ah{k});
            else                    condA(k) = cond(Ah{k});
            end;
        end;            
    end;
end;

% this happens in any cleanup procedure
clear Ah Ainvh bh Ainv A B blk
% these only have effect for lspionline with recinv
clear col deltaAinv den uvT

if level >= 1.5,
    % semi-hard cleanup: cleanup A0, b, PHI but keep parameter histories
    % experiment can be resumed but some computation will have to be re-done
    clear PHI HPHI A0 b OFF
    if level >= 2,      
        % hard cleanup: no evolution replay possible; no resume possible if relying on precomputed PHI
        clear thetah hthetah
    end;
end;
cleanedup = level;
clear level;
save(cleanupdf);
end