function w = llrweights(wfun, d, h)
% Weights for local linear regression (more generally, local weighted regression)
%  W = LLRWEIGHTS(WFUN, D, H)
%  W = LLRWEIGHTS(WFUN, D) 
% Note for finite-extent weight functions, we do NOT zero weights for distances above h, 
% since we assume that the bandwidth is the maximum distance.
% This is done to save computation.
% Inputs:
%   WFUN    - weighting function, see code below for supported functions
%   D       - distances (can be an array of any size, but usually vector)
%   H       - bandwidth (scalar). If not given, maximum of d
% Outputs:
%   W       - weights, same size as d

flag = wfun(1);
if nargin < 3, h = max(d); end;

if flag == 'c',         % 'cubic' (also known as tricubic in the literature)
    w = sqrt((1 - abs(d./h).^3) .^ 3);
elseif flag == 'q',     % 'quad'
    w = 1 - (d./h).^2;  % should in fact be squared and then square root -- so do nothing
elseif flag == 't',     % 'triang'
    w = sqrt(1 - abs(d./h));
elseif flag == 'u',     % 'unif'
    w = ones(size(d));
% elseif flag == 'f',         % 'flat' portion until thresh + tricubic for the rest (debugging option)
%     d = d ./ h;
%     th = .9;
%     d(d <= th) = 0;
%     d(d > th) = (d(d > th) - th) ./ (1-th);
%     w = sqrt((1 - abs(d).^3) .^ 3);
else
    error('Unknown weighting function [%s].', wfun);
end;

% global WW
% global DD
% WW(end+1:end+length(w)) = w;
% DD(end+1:end+length(w)) = d;

% if rand < .05,
% d
% h
% w
% pause
% end;
