function [xplus, rplus, terminal] = dc_linmdp(m, x, u)
% Implements the discrete-time dynamics of a DC motor (linear, no angle
% wrapping, saturation of both state variables)
%  [XPLUS, RPLUS, TERMINAl] = INT_MDP(M, X, U)
%
% This function conforms to the specifications established by SAMPLE_MDP.

xplus = m.A * x + m.B * u;
if isfield(m, 'noise') && ~isempty(m.noise) && m.noise(1) == 'g'
    xplus = xplus + (randn(1, 2) * m.chol)';
end;
xplus = max(-m.maxx, min(m.maxx, xplus));

% Reward
rt = m.rewtype(1);
if rt == 'l',       % 'lqr'
    rplus = - x' * m.Q * x - m.R * u * u;
elseif rt == 'n',   % 'normalized lqr'
    rplus = 1 - (x' * m.Q * x + m.R * u * u) / m.nonnorm_maxr;    
elseif rt == 'g', % gauss
    rplus = -1 + exp(-[x; u]' * m.W * [x; u]);
elseif rt == 's',   % 'shaping' -- LQR + (various types of) shaping
    rplus = - x' * m.Q * x - m.R * u * u + m.gamma * dc_shaping(m, xplus) - dc_shaping(m, x);
elseif (rt == 'b') || (rt == 'B'),   % 'box' or big 'Box'
    rplus = m.zeroreward * all(abs(xplus) <= m.zeroband);
elseif rt == 'w',   % weighted 2-norm
    rplus = 1 - (sqrt(x' * m.Q * x) + sqrt(m.R * u * u)) / m.nonnorm_maxr;    
end;

terminal = 0;       % task is continuing

end
% END FUNCTION int_mpd() RETURNING xplus, rplus ==================================================
