% What experiments to process and in what mode
exp = {'all'};
% exp = {'lqr2rbf7', 'lqr2rbf11', 'lqr2rbf17'};
mode = 'stats';       % 'run', 'sol', 'stats', 'cleanup'

% performance evaluation for all, write report

% main cfg
cfg = struct;
cfg.(mode) = 2;     
cfg.problem = 'dc_problem';
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.maxiter  = 20;
cfg.X0 = 'coarse';      % when evaluating performance, use coarse grid
cfg.mc_eps = 0.1;       % good enough given large LQR scores
cfg.x0 = [pi;0]; cfg.tend = 1.5;       % for replaying
cfg.verb = 0;
cfg.hide = {'rep'};     % these experiments are single, no repetitions
% LQR rew, linear model
lqrcfg = cfg;
lqrcfg.model_params = {'lqr', 'lin'};
% Larger LQR rew Q_2,2, larger gamma, lin model
lqr2cfg = cfg;
lqr2cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'};
lqr2cfg.gamma = .99;
lqr2cfg.maxiter  = 35;
lqr2cfg.infix = 'lqr2';
% cell array of experiment configs
CFG = {};


% ==========================================================
% ======== EXPERIMENTS WITH LARGER Q_2,2 R, gamma=0.99 =====
% ---- TRIANG experiments
cexp = lqr2cfg;
cexp.type     = 'triang'; cexp.miscinfo = 'lqr2t7';
cexp.N        = 7; cexp.M = 5;
cexp.Ns       = [1000, 2500, 5000, 10000, 20000];   % NONINV 500
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type     = 'triang'; cexp.miscinfo = 'lqr2t11';
cexp.N        = 11; cexp.M = 5;
cexp.Ns       = [2500, 5000, 10000, 20000];   % NONINV 1000
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type   = 'triang'; cexp.miscinfo = 'lqr2t17';
cexp.N      = 17; cexp.M = 5;
cexp.Ns     = [10000, 20000]; % NONINV 5000
CFG{end+1} = cexp;

% ---- TRIANG QUADRATIC in u experiments
cexp = lqr2cfg;
cexp.type     = 'triangquad'; cexp.miscinfo = 'lqr2tq7';
cexp.method   = 'rand';
cexp.N        = 7; cexp.M = 5;
cexp.Ns       = [500, 1000, 2500, 5000, 10000, 20000];
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type     = 'triangquad'; cexp.miscinfo = 'lqr2tq11';
cexp.method   = 'rand';
cexp.N        = 11; cexp.M = 5;
cexp.Ns       =  [500, 1000, 2500, 5000, 10000, 20000];
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type     = 'triangquad'; cexp.miscinfo = 'lqr2tq17';
cexp.method   = 'rand';
cexp.N        = 17; cexp.M = 5; 
cexp.Ns       =  [2500, 5000, 10000, 20000];    % 1000 CLOSE TO SINGULAR
CFG{end+1} = cexp;

% ---- RBF experiments
cexp = lqr2cfg;
cexp.type     = 'rbfdisc'; cexp.miscinfo = 'lqr2rbf7';
cexp.N        = 7; cexp.M = 5; cexp.rbfrad = 'avg';
cexp.Ns       = [500, 1000, 2500, 5000, 10000, 20000];
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type     = 'rbfdisc'; cexp.miscinfo = 'lqr2rbf11';
cexp.N        = 11; cexp.M = 5; cexp.rbfrad = 'avg';
cexp.Ns       = [1000, 2500, 5000, 10000, 20000];   % 500 CLOSE TO SINGULAR
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type     = 'rbfdisc'; cexp.miscinfo = 'lqr2rbf17';
cexp.N        = 17; cexp.M = 5;  cexp.rbfrad = 'avg';
cexp.Ns       = [2500, 5000]; % 1000 CLOSE TO SINGULAR; 10000, 20000 way too time-consuming
CFG{end+1} = cexp;

% ---- RBF quad experiments
cexp = lqr2cfg;
cexp.type     = 'rbfquad'; cexp.miscinfo = 'lqr2rq7'; 
cexp.method   = 'rand';
cexp.N        = 7; cexp.M = 5; cexp.rbfrad = 'avg';
cexp.Ns       = [500, 1000, 2500, 5000, 10000, 20000];
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type     = 'rbfquad'; cexp.miscinfo = 'lqr2rq11'; 
cexp.method   = 'rand';
cexp.N        = 11; cexp.M = 5; cexp.rbfrad = 'avg';
cexp.Ns       = [500, 1000, 2500, 5000, 10000, 20000];
CFG{end+1} = cexp;

cexp = lqr2cfg;
cexp.type     = 'rbfquad'; cexp.miscinfo = 'lqr2rq17';
cexp.method   = 'rand';
cexp.N        = 17; cexp.M = 5;  cexp.rbfrad = 'avg';
cexp.Ns       = [1000, 2500, 5000]; % 500 CLOSE TO SINGULAR; 10000, 20000 way too time-consuming
CFG{end+1} = cexp;

% % ==========================================================
% % ===== ORIGINAL EXPERIMENTS WITH Q=diag(5,.01) R=0.01 =====
% 
% % ---- RBF experiments
% cexp = lqrcfg;
% cexp.type     = 'rbfdisc'; cexp.miscinfo = 'rbf7';
% cexp.N        = 7; cexp.M = 5; cexp.rbfrad = 'avg';
% cexp.Ns       = [500, 1000, 2500, 5000, 10000, 20000];
% CFG{end+1} = cexp;
% 
% cexp = lqrcfg;
% cexp.type     = 'rbfdisc'; cexp.miscinfo = 'rbf11';
% cexp.N        = 11; cexp.M = 5; cexp.rbfrad = 'avg';
% cexp.Ns       = [500, 1000, 2500, 5000, 10000, 20000]; % 500 badly conditioned
% CFG{end+1} = cexp;
% 
% cexp = lqrcfg;
% cexp.type     = 'rbfdisc'; cexp.miscinfo = 'rbf17';
% cexp.N        = 17; cexp.M = 5;  cexp.rbfrad = 'avg';
% cexp.Ns       = [2500, 5000]; % 500, 1000 badly conditioned; 10000, 20000 way too time-consuming
% CFG{end+1} = cexp;
% 
% % ---- TRIANG experiments
% cexp = lqrcfg;
% cexp.type     = 'triang'; cexp.miscinfo = 't7';
% cexp.N        = 7; cexp.M = 5;
% cexp.Ns       = [1000, 2500, 5000, 10000, 20000];
% CFG{end+1} = cexp;
% 
% cexp = lqrcfg;
% cexp.type     = 'triang'; cexp.miscinfo = 't11';
% cexp.N        = 11; cexp.M = 5;
% cexp.Ns       = [2500, 5000, 10000, 20000];   % 1000 singular
% CFG{end+1} = cexp;
% 
% cexp = lqrcfg;
% cexp.type   = 'triang'; cexp.miscinfo = 't17';
% cexp.N      = 17; cexp.M = 5;
% cexp.Ns     = [10000, 20000];   % 1000 , 2500, 5000, singular
% CFG{end+1} = cexp;
% 
% % ---- TRIANG QUADRATIC in u experiments
% cexp = lqrcfg;
% cexp.type     = 'triangquad'; cexp.miscinfo = 'tq7';
% cexp.method   = 'rand';
% cexp.N        = 7; cexp.M = 5;
% cexp.Ns       = [500, 1000, 2500, 5000, 10000, 20000];
% CFG{end+1} = cexp;
% 
% cexp = lqrcfg;
% cexp.type     = 'triangquad'; cexp.miscinfo = 'tq11';
% cexp.method   = 'rand';
% cexp.N        = 11; cexp.M = 5;
% cexp.Ns       =  [1000, 2500, 5000, 10000, 20000];  % 500 singular
% CFG{end+1} = cexp;
% 
% cexp = lqrcfg;
% cexp.type     = 'triangquad'; cexp.miscinfo = 'tq17';
% cexp.method   = 'rand';
% cexp.N        = 17; cexp.M = 5; 
% cexp.Ns       =  [2500, 5000, 10000, 20000];    % 1000 singular (=> 500 as well)
% CFG{end+1} = cexp;
% 

% Main experiment loop
for i = 1:length(CFG),
    if any(strcmp(CFG{i}.miscinfo, exp)) || any(strcmp('all', exp)),
        lspiexp(CFG{i});
    end;
end;
