function lspiodc_dupd(exp, mode, c)
% DC motor pendulum. Online, study of influence of dupd

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'ronl'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
% --------------
% main LSPIEXP config
cfg.(mode) = 2;
cfg.startat = 1; cfg.repeat = 5;
% algorithm fixed config
cfg.gamma = .97;
cfg.problem = 'dc_problem';
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model
% misc config (output etc)
cfg.datadir = pwd;
cfg.eps = 0;          % zero convergence threshold (force until real convergence)
% approximator main config
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.datadir = pwd;
% perf eval
cfg.X0 = 'small'; cfg.perf = 1; cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 3;    % for replaying
% plot config
cfg.plottarget = 'screen'; cfg.savefig = 'dupd';
if strcmp(mode, 'plot'), 
    cfg.stats = 1; cfg.plot = {'J'}; 
    cfg.figsize = [450 350];
    cfg.ylim = [-1000 -300];
end;

if any(strcmp(mode, {'stats', 'plot'})), ocfg.verb = 0; else ocfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% --------------
% LSPIONLINE config
ocfg = cfg;
ocfg.alg = 'lspionline';
ocfg.trialtime = 1.5;           % sufficient for stabilization from most (all?) initial states
ocfg.lspi_params.maxtime  = 60*10;          % 10 simulated minutes
ocfg.explordecay = 0.1^(1/ocfg.lspi_params.maxtime);  
ocfg.lspi_params.explor = 1;                % start with this exploration rate
ocfg.lspi_params.dstoretheta = 1;           % store theta once every second
    % decay so that we have 0.1 at the end of the experiment
ocfg.lspi_params.verb = 5;
ocfg.lspi_params.ddisp = 10;
ocfg.lspi_params.dsave = 60;                % save once every minute
ocfg.show = {'dupd', 'rep'};
ocfg.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600]; 
% perf eval
ocfg.forceeval = 1;                         % since no experiment actually converges

% --------------
% LSPI config
bcfg = cfg;
bcfg.alg = 'lspi';
bcfg.show = {'rep'};
bcfg.maxiter = 200;


expout = [];   % dummy

% --------------------------------
% --- ONLINE EXPERIMENTS ---------
% --------------------------------

% ONLINE, RBF discrete
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl';
rexp.N        = 9; rexp.M = 3;
rexp.dupd     = [1 10 100 1000 5000] * 0.005;
% rexp.dupd     = [10] * 0.005; rexp.plotminmax = 1;
rexp.addtocrt = []; rexp.expprefix = 'Online';  % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;


% ONLINE RBF ORTHOPOLY
roexp = ocfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'roonl';
roexp.N        = 9; roexp.M = 2;
roexp.dupd     = [1 10 100 1000 5000] * 0.005;
% roexp.dupd     = [10] * 0.005; roexp.plotminmax = 1;
roexp.addtocrt = []; roexp.expprefix = 'Online ortho';  % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;

% ONLINE RBF POLY
rpexp = ocfg;
rpexp.type     = 'rbfpoly'; rpexp.miscinfo = 'rponl';
rpexp.N        = 9; rpexp.M = 2;
rpexp.dupd     = [1 10 100 1000 5000] * 0.005;
% rpexp.dupd     = [10] * 0.005; rpexp.plotminmax = 1;
rpexp.addtocrt = []; rpexp.expprefix = 'Online poly';  % only effective in stats+plot mode
if checkrunexp(rpexp.miscinfo, exp), expout = lspiexp(rpexp); end;


% --------------------------------
% --- BASELINE EXPERIMENTS -------
% --------------------------------
% BASELINE, offline RBF experiment
rbexp = bcfg;
rbexp.type     = 'rbfdisc'; rbexp.miscinfo = 'rbaseline';
rbexp.N        = 9; rbexp.M = 3;
rbexp.method   = 'randdisc';
rbexp.Ns       = 20000;  % many samples to get close to the best achievable solution
rbexp.addtocrt = expout; rbexp.expprefix = 'Offline'; % only effective in stats+plot mode
rbexp.infix    = 'dupd';
if checkrunexp(rbexp.miscinfo, exp), expout = lspiexp(rbexp); end;

% BASELINE, offline RBF ORTHOPOLY experiment
robexp = bcfg;
robexp.type     = 'rbfortho'; robexp.miscinfo = 'robaseline';
robexp.eps      = 1e-5;     % this one will never actually converge to 0
robexp.N        = 9; robexp.M = 2;
robexp.method   = 'rand';
robexp.Ns       = 20000;  % many samples to get close to the best achievable solution
robexp.addtocrt = expout; robexp.expprefix = 'Offline'; % only effective in stats+plot mode
robexp.infix    = 'dupd';
if checkrunexp(robexp.miscinfo, exp), expout = lspiexp(robexp); end;


end



