function lspiodc_explordecay(exp, mode, c)
% DC motor. Exploration decay study

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 'ronl'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% default (main) cfg
cfg = struct;
cfg.(mode) = 2;
cfg.startat = 1; cfg.repeat = 5;
% algorithm fixed config
cfg.gamma = .97;
cfg.problem = 'dc_problem';
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model% misc config (output etc)
% misc config (output etc)
cfg.datadir = pwd;
cfg.eps = 0;                            % zero convergence threshold (force until real convergence)
% approximator main config
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.datadir = pwd;
% perf eval
cfg.X0 = 'small'; cfg.perf = 1; cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 3;    % for replaying
% plot config
cfg.plottarget = 'screen'; cfg.savefig = 'ed';
if strcmp(mode, 'plot'), 
    cfg.stats = 1; cfg.plot = {'J'}; 
    cfg.figsize = [450 350];
end;
if strcmp(mode, 'onlineperf'), 
%     cfg.ylim = [-2400 -1400];
end;
if any(strcmp(mode, {'stats', 'plot', 'onlineperf'})), cfg.verb = 0; else cfg.verb = 5; end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);

% --------------
% LSPIONLINE config
ocfg = cfg;
ocfg.alg = 'lspionline';
ocfg.trialtime = 1.5; 
ocfg.dupd = 100 * 0.005;                    % 100 steps, different from IPSETUP!!!
ocfg.lspi_params.maxtime  = 60*10;          % 10 simulated minutes
ocfg.lspi_params.explor = 1;                % start with this exploration rate
    % decay so that we have 0.1 at the end of the experiment
ocfg.lspi_params.dstoretheta = 1;           % store theta once every second
ocfg.lspi_params.verb = 5;
ocfg.lspi_params.ddisp = 10;
ocfg.lspi_params.dsave = 60;                % save once every minute
ocfg.show = {'explordecay', 'rep'};
% perf eval
ocfg.donlineperf = [5 10 15 20:10:50 60:15:120 150 180:60:600];
ocfg.forceeval = 1;                         % since no experiment actually converges

expout = [];   % dummy

% ONLINE, RBF discrete experiment
rexp = ocfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'ronl';
rexp.N        = 11; rexp.M = 3;
% these are all the experiments for which data is available
rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 300 600])  0.5^(1/600) 0.8^(1/600)];
% % this is used for performance eval (plotting); the above is too dense and unreadable
% rexp.explordecay = [0.1 .^ (1 ./ [20 50 100 300 600])  0.8^(1/600)];
% rexp.plotminmax = 1;
rexp.addtocrt = []; rexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

% ONLINE, RBF ortho experiment
roexp = ocfg;
roexp.type     = 'rbfortho'; roexp.miscinfo = 'roonl';
roexp.N        = 11; roexp.M = 2;
roexp.explordecay = [0.1 .^ (1 ./ [20 50 100 300 600])  0.5^(1/600) 0.8^(1/600)];
% roexp.plotminmax = 1;
roexp.addtocrt = []; roexp.expprefix = ''; % only effective in stats+plot mode
if checkrunexp(roexp.miscinfo, exp), expout = lspiexp(roexp); end;


end

