
datafile = 'test_edro_rtlspionline_jnt2x3_N5x5x7x3_#2';
load(datafile, 'sys');

% these values seem reasonable (keep within safety bounds, not too fast, etc)
% sys.vectorfield_params = {[.3; .25], .075, 5, 5};

sys = edro_rtproblem('init', sys);

njoints = sys.p / 2;
K = 500;
X = nan(sys.p, K+1);
POS = nan(2, K+1);
% U = nan(sys.q, K+1);
uzero = zeros(sys.q, 1);        % robot is steered 'by hand'
R = nan(1, K+1);

c = sys.vectorfield_params{1};
rad = sys.vectorfield_params{2};

% init figure
figure;
subplot(3, 1, [1 2]);
rectangle('Position', [c(1)-rad c(2)-rad 2*rad 2*rad], 'Curvature', [1 1], 'EdgeColor', 'r', 'LineWidth', 2);
axis equal;
hold on;

tic;
for k = 1 : K,
    x = edro_rtproblem('readx', sys);
    X(:, k) = x;
    POS(:, k) = [- sys.len(2)*sin(X(1, k)) - sys.len(3)*sin(X(1, k)+X(3, k)-pi/2); ...
                sys.len(1) + sys.len(2)*cos(X(1, k)) + sys.len(3)*cos(X(1, k)+X(3, k)-pi/2)];
    if k > 1, 
        % reference position is given by integrating the vector field
        % from pos, for one sample time
        [odet odey] = ode45(sys.vectorfield, [0 sys.Ts], POS(:, k), [], sys.vectorfield_params{:});
        refpos = odey(end, :)';
        % show actual transition and reference
        subplot(3, 1, [1 2]);
        plot(POS(1, [k-1 k]), POS(2, [k-1 k]), 'b', 'Marker', '.');
        plot([POS(1, k-1) refpos(1)], [POS(2, k-1), refpos(2)], 'g', 'Marker', '.');
        subplot(3, 1, 3);
        % plot angle trajectories and safety bounds
        cla;
        plot((0:k-1)*sys.Ts, X(1, 1:k)', 'b');
        hold on;
        plot([0 k-1]*sys.Ts, [0 0] + sys.angsafe(2, 1), 'b--');
        plot([0 k-1]*sys.Ts, [0 0] + sys.angsafe(2, 2)), 'b--';
        plot((0:k-1)*sys.Ts, X(3, 1:k)', 'r');
        plot([0 k-1]*sys.Ts, [0 0] + sys.angsafe(3, 1), 'r--');
        plot([0 k-1]*sys.Ts, [0 0] + sys.angsafe(3, 2), 'r--');
        drawnow;
    end;
    while toc < sys.Ts, end;          % wait for the next sample
    tic;
end;

% read final state and reward
X(:, k+1) = edro_rtproblem('readx', sys);
% R(:, k+1) = sys.fun(sys, X(:, k), uzero, X(:, k+1));

% h.t = sys.Ts .* (0:K); h.x = X; h.u = U; h.r = R;
% plothistory(h);

sys = edro_rtproblem('close', sys);
