% File for running HIV control experiments with optimistic planning

% model config
MCFG = struct;
MCFG.filter = 'none';
MCFG.rewtype = 'normadams';  % normalized!
ODECFG = struct;

CFG = struct;
CFG.planparams.U = flat({[0, 0.7], [0, 0.3]});
CFG.trials = 1;
CFG.reset = 'unhealthy';
CFG.maxsteps = 800/5;
CFG.verb = 3;
CFG.datadir = pwd;


%% Deterministic
mcfg = MCFG; 
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opd';
% cfg.planparams.criterion = 'contrib'; % NOTE contrib has no meaning in OPD!! (but this line was
% commented out so it's OK)
% cfg.planparams.actsel = 'util';
cfg.planparams.n = 1000;
cfg.datafile = 'hiv_op_det_opd_n1000';

cfg.run = 1;
genmpc(cfg);


%% Stochastic, OPSS
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opss';
cfg.planparams.criterion = 'contrib';
cfg.planparams.actsel = 'util';
cfg.planparams.L = 4;
cfg.planparams.n = 3000;
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_opss_n3000_p05g09p05g11_T1100';

cfg.run = 1;
genmpc(cfg);


%% Stochastic, OPSS with memory reuse
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opss';
cfg.planparams.criterion = 'contrib';
cfg.planparams.actsel = 'util';
cfg.planparams.L = 4;
cfg.planparams.n = 1000;
cfg.planparams.usemem = 1;
cfg.planparams.mem_maxsize = 1;     % 1 * tree size
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_opss_usemem_n1000_memmaxsize1_p05g09p05g11_T1100';

cfg.run = 1;
genmpc(cfg);

%% Stochastic, uniform
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opss';
cfg.planparams.criterion = 'unif';
cfg.planparams.actsel = 'util';
cfg.planparams.L = 4;
cfg.planparams.n = 3000;
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_unif_n3000_p05g09p05g11_T1100';

cfg.run = 1;
genmpc(cfg);

%% Stochastic, OLOP
mcfg = MCFG;
mcfg.stoch_gain = [1.1 .9];
mcfg.stoch_prob = [.5 .5];
odecfg = ODECFG;

cfg = CFG;
cfg.problem = 'hiv_problem';
cfg.model_params = {mcfg, odecfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'olop';
cfg.planparams.n = 3000 * 4 * size(CFG.planparams.U, 2);
cfg.planparams.verb = 1;
cfg.maxsteps = 1100/5;
cfg.datafile = 'hiv_op_stoch_olop_n48000_p05g09p05g11_T1100';

cfg.run = 1;
genmpc(cfg);
