function dx = invpend_cart_trans(t, x, m, u)
%  Implements the transition function of the cart-inverted pendulum system, using an infinite
%  track. This is the version used in the LSPI paper of Lagoudakis and Parr, 2003
%
%  Parameters:
%   T           - the current time. Ignored, added for compatibility with
%               the ODEXX function style.
%   X           - the current state
%   M           - the model structure specifying physical params
%   U           - the command
%
%  Returns:
%   DX          - the derivative of the state
%

theta = x(1);
dx(2,1) = (m.g*sin(theta) - m.alphaml * x(2)*x(2) * sin(2*theta)/2 - m.alpha*cos(theta) * u) ...
    / (4*m.l/3 - m.alphaml*cos(theta)^2);
dx(1,1) = x(2);

% end INVPEND_TRANS returning DX
