% Overactuated inverted pendulum (no swing is necessary to stabilize)
function lspiip_gauss(exp, mode, c)
% Real inverted pendulum. Comparison of approximators

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = 't*'; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% main cfg
cfg = struct;
cfg.(mode) = 2;
cfg.problem = 'invpend_problem';
% this should not give convergence problems, while still giving a time horizon for a good solution
cfg.gamma = 0.97;
cfg.maxiter  = 100;
cfg.repeat = 1;
cfg.Mpoly = [2 3 4];
cfg.xspace = 'log'; cfg.uspace = 'log';
cfg.Ns = 20000;                 % baseline for adaptive Ns
cfg.adaptiveNs = 1;
cfg.model_params = {'gauss', 'type=rot', 'Ts=0.02'};
cfg.show = {'gamma', 'rep'};
cfg.datadir = pwd;
% perf eval
cfg.perf = 1;
cfg.infix = 'gauss';
cfg.X0 = 'coarse';
cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 5;    % for replaying
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'J', 'iter'}; 
    cfg.xlim = [2 5];
    cfg.xtick = 2:5;
    cfg.ylim_matrix = [-222 -211; 0 100];
    cfg.figsize = [450 350];
end;

% install any non-default config; preserve the default
CFG = cfg; cfg = parseconfig(c, cfg);
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 5; end;

expout = [];    % dummy

toexp_Ns = cfg;          % ortho
toexp_Ns.type     = 'triangortho'; toexp_Ns.miscinfo = 'toNs';
toexp_Ns.N        = 11; toexp_Ns.M = cfg.Mpoly; 
toexp_Ns.orthotype = 'cheby';
toexp_Ns.method   = 'rand'; 
toexp_Ns.addtocrt = 0; toexp_Ns.expprefix = 'ortho,';     % only effective in stats+plot mode
if checkrunexp(toexp_Ns.miscinfo, exp), expout = lspiexp(toexp_Ns); end;

% % Triang approx
% tpexp_Ns = cfg;      % poly
% tpexp_Ns.type     = 'triangpoly'; tpexp_Ns.miscinfo = 'tpNs';
% tpexp_Ns.N        = 11; tpexp_Ns.M = cfg.Mpoly;
% tpexp_Ns.method   = 'rand';
% tpexp_Ns.usesamples = toexp_Ns;
% tpexp_Ns.addtocrt = 0; tpexp_Ns.expprefix = 'poly,';     % only effective in stats+plot mode
% if checkrunexp(tpexp_Ns.miscinfo, exp), expout = lspiexp(tpexp_Ns); end;

% texp_Ns = cfg;       % disc
% texp_Ns.type     = 'triang'; texp_Ns.miscinfo = 'tNs';
% texp_Ns.N        = 11; texp_Ns.M = [3 5];
% texp_Ns.method   = 'randdisc';
% texp_Ns.addtocrt = expout; texp_Ns.expprefix = 'discrete,'; % only effective in stats+plot mode
% if checkrunexp(texp_Ns.miscinfo, exp), expout = lspiexp(texp_Ns); end;

% RBF approx
% rpexp_Ns = cfg;      % poly
% rpexp_Ns.type     = 'rbfpoly'; rpexp_Ns.miscinfo = 'rpNs';
% rpexp_Ns.N        = 11; rpexp_Ns.M = cfg.Mpoly;
% rpexp_Ns.method   = 'rand';
% rpexp_Ns.usesamples = toexp_Ns;
% rpexp_Ns.addtocrt = 0; rpexp_Ns.expprefix = 'poly,'; % only effective in stats+plot mode
% if checkrunexp(rpexp_Ns.miscinfo, exp), expout = lspiexp(rpexp_Ns); end;

roexp_Ns = cfg;            % ortho
roexp_Ns.type     = 'rbfortho'; roexp_Ns.miscinfo = 'roNs';
roexp_Ns.N        = 11; roexp_Ns.M = cfg.Mpoly; 
roexp_Ns.orthotype = 'cheby';
roexp_Ns.method   = 'rand';
roexp_Ns.usesamples = toexp_Ns;
roexp_Ns.addtocrt = 0; roexp_Ns.expprefix = 'ortho,'; % only effective in stats+plot mode
if checkrunexp(roexp_Ns.miscinfo, exp), expout = lspiexp(roexp_Ns); end;

% rexp_Ns = cfg;       % disc
% rexp_Ns.type     = 'rbfdisc'; rexp_Ns.miscinfo = 'rNs';
% rexp_Ns.N        = 11; rexp_Ns.M = [3 5];
% rexp_Ns.method   = 'randdisc';
% rexp_Ns.usesamples = texp_Ns;
% rexp_Ns.addtocrt = expout; rexp_Ns.expprefix = 'discrete,'; % only effective in stats+plot mode
% if checkrunexp(rexp_Ns.miscinfo, exp), expout = lspiexp(rexp_Ns); end;
