% What experiments to process and in what mode
exp = {'all'};
mode = 'cleanup';       % 'run', 'sol', 'stats', 'cleanup'

% main cfg
cfg = struct;
cfg.(mode) = 2;
cfg.problem = 'invpend_problem';
% this should not give convergence problems, while still giving a time horizon for a good solution
cfg.gamma = 0.91;
cfg.maxiter  = 100;
cfg.repeat = 3;
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.Ns = 10000;
% cfg.samesamples = 1;
cfg.model_params = {'lqr', 'type=rot maxu=2', 'Ts=0.02'};
cfg.infix = 'u2h0nil';
cfg.show = {'gamma', 'rep'};
cfg.datadir = pwd;
% perf eval
cfg.perf = 1;
cfg.X0 = 'coarse';
cfg.mc_eps = 0.1; 
% replay
cfg.x0 = [pi;0]; cfg.tend = 5;    % for replaying
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'J', 'iter'};
%     cfg.plottarget = 'latex'; 
    cfg.savefig = 'iph0fz';
    cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\';
    cfg.figsize = [450 350];
end;
if any(strcmp(mode, {'stats', 'plot'})), cfg.verb = 0; else cfg.verb = 3; end;

% Config for using PD policy (discounted LQR controller)
hcfg_fc = cfg;
hcfg_fc.lspi_params.h0 = @fuzzyqi_h;
hcfg_fc.lspi_params.h0args = {fuzzyqi_prepareh('fzip_fine_box', 1)};  % 0 = no interpolation;
hcfg_fc.infix = 'u2h0fc';

expout = [];        % reset figure additions
% TRIANGULAR discrete / min time policy
thexp_fc = hcfg_fc;
thexp_fc.type     = 'triang'; thexp_fc.miscinfo = 'th_fc';
thexp_fc.N        = 11; thexp_fc.M = 3;
thexp_fc.method   = 'randdisc';
thexp_fc.addtocrt = 0; thexp_fc.expprefix = 'min time,';     % only effective in stats+plot mode
if checkrunexp(thexp_fc.miscinfo, exp), expout = lspiexp(thexp_fc); end;
% TRIANGULAR discrete
texp = cfg;
texp.type     = 'triang'; texp.miscinfo = 't_nil';
texp.N        = 11; texp.M = 3; 
texp.method   = 'randdisc';
texp.addtocrt = expout; texp.expprefix = 'no prior,';     % only effective in stats+plot mode
texp.usesamples = thexp_fc;
if checkrunexp(texp.miscinfo, exp), expout = lspiexp(texp); end;

expout = [];        % reset figure additions
% TRIANGULAR quad min time policy
tphexp_fc = hcfg_fc;
tphexp_fc.type     = 'triangpoly'; tphexp_fc.miscinfo = 'tph_fc';
tphexp_fc.N        = 11; tphexp_fc.M = 2;
tphexp_fc.method   = 'rand';
tphexp_fc.addtocrt = 0; tphexp_fc.expprefix = 'min time,';     % only effective in stats+plot mode
if checkrunexp(tphexp_fc.miscinfo, exp), expout = lspiexp(tphexp_fc); end;
% TRIANGULAR quad
tpexp = cfg;
tpexp.type     = 'triangpoly'; tpexp.miscinfo = 'tp_nil';
tpexp.N        = 11; tpexp.M = 2;
tpexp.method   = 'rand';
tpexp.usesamples = tphexp_fc;
tpexp.addtocrt = expout; tpexp.expprefix = 'no prior,';     % only effective in stats+plot mode
if checkrunexp(tpexp.miscinfo, exp), expout = lspiexp(tpexp); end;

expout = [];        % reset figure additions
% -------- RBF experiments, regular inversion
% RBF discrete min time policy
rhexp_fc = hcfg_fc;
rhexp_fc.type     = 'rbfdisc'; rhexp_fc.miscinfo = 'rh_fc';
rhexp_fc.N        = 11; rhexp_fc.M = 3;
rhexp_fc.method   = 'randdisc';
rhexp_fc.usesamples = thexp_fc;
rhexp_fc.addtocrt = 0; rhexp_fc.expprefix = 'min time,';     % only effective in stats+plot mode
if checkrunexp(rhexp_fc.miscinfo, exp), expout = lspiexp(rhexp_fc); end;
% RBF discrete
rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r_nil';
rexp.N        = 11; rexp.M = 3; 
rexp.method   = 'randdisc';
rexp.usesamples = thexp_fc;
rexp.addtocrt = expout; rexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;

expout = [];        % reset figure additions
% RBF poly min time policy
rphexp_fc = hcfg_fc;
rphexp_fc.type     = 'rbfpoly'; rphexp_fc.miscinfo = 'rph_fc';
rphexp_fc.N        = 11; rphexp_fc.M = 2;
rphexp_fc.method   = 'rand';
rphexp_fc.usesamples = tphexp_fc;
rphexp_fc.addtocrt = 0; rphexp_fc.expprefix = 'min time,';     % only effective in stats+plot mode
if checkrunexp(rphexp_fc.miscinfo, exp), expout = lspiexp(rphexp_fc); end;
% RBF poly
rpexp = cfg;
rpexp.type     = 'rbfpoly'; rpexp.miscinfo = 'rp_nil';
rpexp.N        = 11; rpexp.M = 2;
rpexp.method   = 'rand';
rpexp.usesamples = tphexp_fc;
rpexp.addtocrt = expout; rpexp.expprefix = 'no h_0,';     % only effective in stats+plot mode
if checkrunexp(rpexp.miscinfo, exp), expout = lspiexp(rpexp); end;



