% Study of grid Q-iteration results for the inverted pendulum setup
% Very fine grids over the action space are tried, up to 601 points/axis. The action space is
% discrete and equal to {-max, 0, max}.
%
% The goal of these experiments were to try answering Referee 2's question on the ICML09 LSPI
% submission, asking whether discretization yields good results on this problem.
% The answer is: it does
% 
% This directory also contains the policies found by grid QI in PNG format


% Common config (model etc.)
ccfg = struct;
ccfg.problem = 'ipsetup_problem';
ccfg.model_params = {[], 'type=ip maxu=3', 'Ts=0.005'};
ccfg.gamma = .98;
ccfg.datadir = pwd;
ccfg.eps = 1e-3;
ccfg.maxiter = 1000;
ccfg.initstepdisp = .05;

% create a dummy model
dm = ipsetup_problem('model', ccfg.model_params{:});

% setup experiments
fine100.filename = 'gridqis_N101M3';
fine100.N = 101;
fine100.M = 3;

fine300.filename = 'gridqis_N301M3';
fine300.N = 301;
fine300.M = 3;

fine600.filename = 'gridqis_N601M3';
fine600.N = 601;
fine600.M = 3;

experiments = {fine100, fine300, fine600};

% run experiments and examine results
for i = 1:length(experiments),
    expcfg = experiments{i};
    if ~exist([expcfg.filename '.mat'], 'file'),
        cfg = ccfg;
        cfg.run = 1;
        cfg.datafile = expcfg.filename;
        cfg.xgrids = symequidgrid([expcfg.N expcfg.N], dm.maxx); 
        cfg.ugrids = {symequidgrid(expcfg.M, dm.maxu)};
        gridqi(cfg);
    end;
    
    % evaluate results: score on 'small' grid of initial states
    [mJ, J, K] =  mc_gridqi(expcfg.filename, 'small', 0.1);
    disp(sprintf('Score for %d=%f', i, mJ));  
        % results: 
        % Score for 1=-2164.423895
        % Score for 2=-1628.392788
        % Score for 3=-1475.113959 (near-optimal)
    
    % evaluate results: policy
    dc_plot(sprintf('gridh datafile=%s plottarget=png savefig=%s', ...
        expcfg.filename, [expcfg.filename '_h']));
end;