function figh = ipconsstudy(cfg)
% Consistency study of fuzzyqi for the IP setup motor

% CFG.plottarget = 'latex';
CFG.experiments = {'lqr', 'shapbox'};

% CFG.N = 3:3:41;       % quick res study
% CFG.M = 3:2:7;
CFG.N = 3:1:41;       % more detailed res study
CFG.M = 3:2:15;
% CFG.N = [40 41];       % quick res study
% CFG.M = [13 15];

CFG.datadir = 'd:\work\m\approxrl\systems\invpend_setup\ipconsstudy\';
CFG.savedir = 'D:\Work\tex\presentations\2008-04-09_dcsc-colloq\img\';
CFG.problem = 'ipsetup_problem';
CFG.X0 = 'coarse2';
CFG.mc_eps = 0.1;
CFG.gamma = 0.98;
CFG.plottarget = 'beamer';      % use 'screen' to not overwrite figures
CFG.plot3fun = @mesh;
CFG.view = {-21, 30};
CFG.figsize = [500 400];

if nargin < 1, cfg = struct; end;
if ischar(cfg),  cfg = str2cfg(cfg, fieldnames(CFG)); end;
cfg = checkparams(cfg, CFG);
cfg.grayscale = grayscalefromconfig(cfg);
cfg.xlim = [cfg.N(1) cfg.N(end)];
cfg.ylim = [cfg.M(1) cfg.M(end)];

if ischar(cfg.experiments), cfg.experiments = {cfg.experiments}; end;

% save crt dir, change to datadir
crtdir = pwd;
cd(cfg.datadir);

figh = [];

% main reward plot config
rpcfg.rew = 1;
rpcfg.problem = 'ipsetup_problem';
rpcfg.savedir = cfg.savedir;
% rpcfg.savefig = 'lqr_rew';
rpcfg.plottarget = cfg.plottarget;
rpcfg.slice = [NaN, NaN, 0];      % 0-action slice
rpcfg.gridres = 60;
rpcfg.zlim = [-300 20];
rpcfg.figsize = cfg.figsize;

% LQR basic options are needed as a baseline by other experiments
lqrcfg = cfg;
lqrcfg.model_params = {'lqr'};  % default Q and R
lqrcfg.dfinfix = 'lqr';

% ------------------
% LQR reward experiments
if any(strcmp(cfg.experiments, 'lqr')),
    
    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'lqr'};   % default Q and R
    rewcfg.savefig = 'lqr_rew';
    figh = [figh dc_plot(rewcfg)];

    % run
    rcfg = lqrcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    % plot perf
    scfg = lqrcfg; 
    scfg.datafile = 'score_lqr';
    scfg.usedatafile = 'score_lqr';
    scfg.perf = 1;
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqr';
%     scfg.zlim = [-2800 -2000];
    figh = [figh qiresstudy(scfg)];
    
   % zoom-in on perf
    scfg.N = 13:41;
    scfg.xlim = [13 41];
    scfg.savefig = 'score_lqr_zoomin';
%     scfg.zlim = [-2220 -2100];
    figh = [figh qiresstudy(scfg)];
    
end;

% ------------------
% Box shaping reward experiments
if any(strcmp(cfg.experiments, 'shapbox')),
    
    % reward
    rewcfg = rpcfg;
    rewcfg.model_params = {'shapbox'};
    rewcfg.savefig = 'shapbox_rew';
    rewcfg.gridres = 60;
    rewcfg.enhance = {[-pi/4 pi/4], [-2*pi 2*pi], []};
%     rewcfg.enhfactor = 4;
%     rewcfg.restrictx = [pi/4+pi/12, 4*pi+pi/3];
    figh = [figh dc_plot(rewcfg)];

    shapboxcfg = cfg;
    shapboxcfg.model_params = {'shapbox'};
    shapboxcfg.dfinfix = 'shapbox';
    
    % run
    rcfg = shapboxcfg; rcfg.run = 1; rcfg.perf = 0;
    qiresstudy(rcfg);
    
    % plot perf w/ LQR reward fun
    scfg = shapboxcfg;
    scfg.datafile = 'score_shapbox_lqr';
    scfg.usedatafile = 'score_shapbox_lqr';
    scfg.perf = 1; scfg.userew = 'lqr';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_shapbox_lqr';
%     scfg.zlim = [-2800 -2000];
    figh = [figh qiresstudy(scfg)];

    % zoom-in
    scfg.N = 13:41;
    scfg.xlim = [13 41];
    scfg.savefig = 'score_shapbox_lqr_zoomin';
%     scfg.zlim = [-2220 -2100];
    figh = [figh qiresstudy(scfg)];
    
%     % plot perf w/ original rew fun
%     scfg = shapboxcfg; 
%     scfg.datafile = 'score_shapbox';
%     scfg.perf = 1;
%     scfg.plottarget = cfg.plottarget;
%     scfg.savefig = 'score_shapbox';
%     scfg.zlim = [-400 -150];   
%     figh = [figh qiresstudy(scfg)];

    % 2-D comparison
    scfg = lqrcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.addtocrt = 0; scfg.legendpostfix = 'quadratic reward';
    scfg.lsty = {'-', 'Color', 'b', 'Marker', 'o', 'MarkerSize', 3, 'LineWidth', 1.3};
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsshapbox';
    qiresstudy(scfg);

    clear scfg;
    scfg = shapboxcfg; scfg.view = []; scfg.ylim = [];
    scfg.datafile = 'score_shapbox_lqr';
    scfg.userew = 'lqr';
    scfg.perf = 1; scfg.avgM = 1;
    scfg.lsty = {'-', 'Color', 'r', 'Marker', 'X', 'MarkerSize', 5, 'LineWidth', 1.3};
    scfg.addtocrt = 1; scfg.legendpostfix = 'discontinuous reward';
    scfg.plottarget = cfg.plottarget;
    scfg.savefig = 'score_lqrvsshapbox';
    figh = [figh qiresstudy(scfg)];
    
end;

% change dir to orig
cd(crtdir)
    
