%% Optimistic planning experiments for the inverted pendulum
function op_ip(mode, expids, custombcfg, customacfg)
% 19 Apr 2011: renamed "clopfinite" files manually to "opss", changed algorithm name below to "opss"
% -- seems to work


%% General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'genmpc';
BCFG.verb = 5;
BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'problem', 'planalg', 'X', 'planparamsn'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% algorithm
ACFG = struct;
ACFG.problem = 'ipsetup_problem';
ACFG.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.6,.4] stoch_gain=[1,.7]', 'Ts=0.05'};
ACFG.online = 0;
ACFG.gamma = 0.95;
ACFG.planalg = 'opss';
ACFG.planparams.L = 2;
ACFG.planparams.actsel = 'util';
ACFG.planparams.U = [-3 0 3];
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 50;
ACFG.reset = [-pi, 0]';
% visualization
ACFG.verb = 3;
ACFG.datadir = 'd:\work\m\approxrl\systems\invpend_setup\op_ip';
ACFG.X = 'coarse';  % results also available for "coarser"

cd(ACFG.datadir); 

% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            bcfg = copyfields(custombcfg, bcfg); 
            acfg = copyfields(customacfg, acfg);
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            elseif any(strcmp(mode, {'regret', 'exectime', 'depth'})), 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

%% Near-optimal solution: fuzzyQI -- run if not present

% REMARK: This file was run while fuzzyQI had the bug where all F values were zero if 'term=zero'. 
% Nevertheless this file is OK since it leaves 'term' to 'ignore'.

cfg = ACFG;
dummymodel = ipsetup_problem('model', cfg.model_params{:});
cfg.xgrids = symequidgrid(201, dummymodel.maxx);
cfg.ugrids = {cfg.planparams.U};
cfg.maxiter = 1000;
cfg.eps = 1e-5;
cfg.datafile = 'fzqi_ip_nearopt_p04g07_Ts005';

% add near-optimal solution to default config if need to compute regret
if any(strcmp('nearopt', expids)) || (strcmp(mode, 'regret') && ~exist([cfg.datafile '.mat'], 'file')),
    cfg.run = 1;
    fuzzyqi(cfg);
end;
if strcmp(mode, 'regret'),      % construct optimal solution in format required to evaluate simple regret
    [approx, theta] = fuzzyqi(['approx datafile=' cfg.datafile]);
    ACFG.hstar = @(x) approx.h(approx, theta, x);
    ACFG.Qstar = @(x, u) approx.q(approx, theta, x, u); 
end;


%% Offline, regular expansion criterion //  100-1000 TUD Desktop; 50 F6Ve laptop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'offlcontrib';
bcfg.legendprefix = 'OPSS';

acfg = ACFG;
acfg.planparams.n = [50 100:100:1000];
acfg.planparams.criterion = 'contrib';

execbatchexp(bcfg.idinfix);


%% Offline, uniform expansion // 100-1000 TUD Desktop; 50 F6Ve laptop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'offlunif';
bcfg.legendprefix = 'Uniform';

acfg = ACFG;
acfg.planparams.n = [50 100:100:1000];
acfg.planparams.criterion = 'unif';

execbatchexp(bcfg.idinfix);

%% Offline, smallest-depth criterion // F6Ve laptop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'offlsmallest';
bcfg.legendprefix = 'OPSS smallest';

acfg = ACFG;
acfg.planparams.n = 100:100:1000;
acfg.planparams.criterion = 'smallest';

execbatchexp(bcfg.idinfix);

%% Offline, constrained contrib criterion // F6Ve laptop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'offlxcontrib';
bcfg.legendprefix = 'OPSS x-contrib';

acfg = ACFG;
acfg.planparams.n = 100:100:1000;
acfg.planparams.criterion = 'xcontrib';

execbatchexp(bcfg.idinfix);

%% Online, regular expansion criterion  // TUD Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'onlcontrib';
% bcfg.legendprefix = 'Optimistic';

acfg = ACFG;
acfg.online = 1;
acfg.planparams.n = 600;
acfg.planparams.criterion = 'contrib';

execbatchexp(bcfg.idinfix);


%% Online, uniform expansion criterion  // TUD Desktop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'onlunif';
% bcfg.legendprefix = 'Uniform';

acfg = ACFG;
acfg.online = 1;
acfg.planparams.n = 600;
acfg.planparams.criterion = 'unif';

execbatchexp(bcfg.idinfix);



%% OLOP, Open-Loop Optimistic Planning //  100-1000 TUD Desktop; 50 F6Ve laptop

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.id = 'olop';   % note we don't use this as an infix -- it already appears as algorithm name
bcfg.legendprefix = 'OLOP';
bcfg.repeat = 10;
bcfg.plotconf = 0;  % the interval is so tight it's useless
% transform input to align it with the OPSS "n"
bcfg.intransform = @(val) round(val ./ ACFG.planparams.L ./ length(ACFG.planparams.U));

acfg = ACFG;
acfg.planalg = 'olop';
acfg.planparams.n = [50 100:100:1000] * ACFG.planparams.L * length(ACFG.planparams.U);
acfg.planparams.exptype = 'one';

execbatchexp(bcfg.id);

end
