%% Optimistic planning experiments for the inverted pendulum
function opmdplearn_stochip(mode, expids, custombcfg, customacfg, customplanparams)

%% General config
% clear all; close all;

% Stochastic: batch experiment
BCFG = struct;
BCFG.alg = 'genmpc';
BCFG.verb = 5;
BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'problem', 'planalg', 'planparamsn'};
BCFG.startat = 1; BCFG.repeat = 20;
BCFG.addtocrt = []; BCFG.plotindex = 0;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute
% BCFG.dryrun = 1;
%
% Stochastic: algorithm
ACFG = struct;
ACFG.problem = 'ipsetup_problem';
ACFG.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.6,.4] stoch_gain=[1,.7]', 'Ts=0.05'};
ACFG.online = 1;
ACFG.gamma = 0.95;
ACFG.planalg = 'opmdp';
ACFG.planparams.L = 2;
ACFG.planparams.U = [-3 0 3];
ACFG.trials = 1;
ACFG.maxsteps = 80;
ACFG.reset = [-pi; 0];
ACFG.verb = 3;
ACFG.datadir = 'D:\Work\m\approxrl\systems\invpend_setup\opmdplearn_stochip';

% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;
if nargin < 5, customplanparams = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)) || any(strcmp(expids, {'all', 'any'})),  % if experiment is requested
            bcfg = parseconfig(custombcfg, bcfg); 
            acfg = parseconfig(customacfg, acfg);
            acfg.planparams = parseconfig(customplanparams, acfg.planparams);
            cd(acfg.datadir); 
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            elseif any(strcmp(mode, {'regret', 'exectime', 'plantime', 'depth', 'return'})), 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

%% Baseline: no data reuse

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'base';
bcfg.legendprefix = 'OPMDP';

acfg = ACFG;
acfg.planparams.n = [50 100 200 400];

execbatchexp(bcfg.idinfix);

%% HOLOP -- competing algorithm

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'holop';
bcfg.legendprefix = 'HOLOP';
bcfg = rmfield(bcfg, 'in');
bcfg.in.id = 'planparamsn';
bcfg.in.transform = @(x) x./5;      % to be compatible with "expansion budget" for the other algos

bcfg.startat = 1;
bcfg.idfields = {'alg', 'problem', 'planparamsn', 'planparamsK', 'rep'};

acfg = ACFG;
acfg.planalg = 'holop';
acfg.planparams.n = [50 100 200 400] * 5; % each node expansion in OPMDP costs 5 simulations
acfg.planparams.K = [5 10 15 20 25 30];
acfg.planparams.detact = 0;

execbatchexp(bcfg.idinfix);

%% Lipschitz

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'lip';
bcfg.legendprefix = 'L-OPMDP with Lipschitz';
bcfg.in = 'planparamsn';
bcfg.idfields = {'alg', 'problem', 'planalg', 'planparamsn', 'planparamslipschitz'};

acfg = ACFG;
acfg.planparams.n = [50 100 200 400];
acfg.planparams.blearn = @learn_lipschitz;
acfg.planparams.mem_maxsize = 0;  % no limit
acfg.planparams.lipschitz = [0.01 0.1 0.5 1 5];

execbatchexp(bcfg.idinfix);

%% L-LSSVR

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'lsvr';
bcfg.legendprefix = 'L-OPMDP with L-LSSVR';
bcfg.in = 'planparamsn';
bcfg.idfields = {'alg', 'problem', 'planalg', 'planparamsn', 'planparamsKnn', 'planparamssvrsigma', 'planparamssvrgamma'};

acfg = ACFG;
acfg.planparams.n = [50 100 200 400];
acfg.planparams.blearn = @learn_llssvr;
acfg.planparams.Snn = [1;1];        % fixed to unit weight
acfg.planparams.Knn = [5 10 15];
acfg.planparams.svrsigma = [0.1 0.2 0.3 0.4];
acfg.planparams.svrgamma = [500 750 1000];

execbatchexp(bcfg.idinfix);



end
