function out = ddi_problem(what, varargin)
% Bicycle problem setup.
%   OUT = DDI_PROBLEM(MODE, [PARAM1, [PARAM2, ...]])
% This function conforms to the specifications established by SAMPLE_PROBLEM.
%
% When MODE = 'model':
%   OUT = DDI_PROBLEM(MODE, [REW]])
% where:
%   REW         - type of reward function. Default 'quad'. Possible values:
%       'quad'  - quadratic reward in x and xdot.
%       'lin'   - linear reward in xdot when x is terminal
%       'disc'  - discontinuous reward in xdot (0-centered range of positive reward) 
%               when x is terminal 
%
% When MODE = 'ce':
%   OUT = DDI_PROBLEM(MODE, [X0TYPE]])
% where:
%   X0TYPE      - specify type of interesting state -- see code for values and default
%

maxp = 10; maxv = 5; maxu = 2;
maxx = [maxp; maxv];

xgrids = {-maxp:.25:maxp, -maxv:.25:maxv};
ugrids = {-maxu:.1:maxu};
x0 = (2*rand(2, 1)) .* maxx - maxx;     % default state for replays
gamma = 0.98;
switch what
        
    case 'model'
        m.maxx = maxx;
        m.maxu = maxu;
        m.p = 2; 
        m.q = 1;
        m.det = 1;
        
        Ts = .5;
        A=[0 1; 0 -.2]; B=[0; 1]; C=eye(2); D=[0 ;0];
        A=[0 1; 0 -.2]; B=[0; 1]; C=eye(2); D=[0 ;0];
        sysC=ss(A,B,C,D);
        sysD=c2d(sysC,Ts);
        m.A = sysD.a;
        m.B = sysD.b;

        % reward config
        m.Q = eye(2); 
        m.R = 0.3;
        
        m.fun = @cart_mdp;      % replaces the obsolete ddi_mdp
        m.Ts = Ts;               % sample time unimportant
        
        out = m;

    case 'fuzzy'
        % these are required by the algorithms
        cfg.xgrids = xgrids;
        cfg.ugrids = ugrids;
        cfg.x0 = x0;
        cfg.gamma = gamma;

        out = cfg;

    case 'grid'
        % these are required by the algorithms
        cfg.xgrids = xgrids;
        cfg.ugrids = ugrids;
        cfg.x0 = x0;
        cfg.gamma = gamma;

        out = cfg;
        
        
        % for CE PPS
        % for CE fuzzy QI
%         cfg.Np = [9 9];
end;


% END doubleint_problem(), RETURNING out ====================================