%% config
clear all;
savedir = 'd:\Work\tex\techrep_preliminary\switched\img\';
B1 = [0 1; -10 -1]; B2 = [0 1; -.1 -.5];
Ts = 0.5;
A1 = expm(B1 * Ts); A2 = expm(B2 * Ts); 
mcfg.A = {A1, A2};
mcfg.Q = eye(2);
mcfg.maxx = [30; 30];
mcfg.rewtype = 'safe';

x0 = [1; 1];
% dummym = switchedlin_problem('model', mcfg);

ccfg = struct;
ccfg.problem = 'switchedlin_problem';
ccfg.gamma = .98;
ccfg.planparams.U = [1 2];
ccfg.planparams.n = 2000;
ccfg.verb = 3;

% we can handle things in two ways: lift the system to a dwelltime-multi-step variant
cfg1 = ccfg; 
cfg1.maxsteps = 100;
cfg1.planalg = 'opd'; 
mcfg.dwelltime = 6; 
mcfg.Ts = mcfg.dwelltime * Ts;
cfg1.model_params = {mcfg};
% or use a constrained optimal control algorithm, in which case no change is made to the model
cfg2 = ccfg; 
cfg2.maxsteps = 100 * 6;
cfg2.planalg = 'opddwelltime'; 
cfg2.dwelltime = 6;
mcfg.Ts = Ts;
mcfg.dwelltime = 1; 
cfg2.model_params = {mcfg};

%% multistep variant -- run OPD with no initial mode fixed

cfg1.run = 1;
cfg1.reset = x0;
cfg1.datafile = 'swlin_dwelltime6_lifted';
genmpc(cfg1);

%% process and plot
h = genmpc(['plottrial datafile=swlin_dwelltime6_lifted']);
cost = 0;
for i = 1:length(h.x)-1,
    x = h.x(:, i);
    for j = 1:6, cost = cost + x' * mcfg.Q * x; x = mcfg.A{h.u(i)} * x; end;
    if any(x ~= h.x(:, i+1)), fprintf('State mismatch %g\n', max(abs(x - h.x(:, i+1)))); end;
end;
cost
figure; 

subplot(211);
stairs(h.t, h.u, '-k'); ylabel('Mode'); ylim([0.9 2.1]);
subplot(212);
plot(h.t, h.x(1, :), '-k'); hold on;
plot(h.t, h.x(2, :), '--k');
xlabel('t [s]'); ylabel('States'); legend('x_1(t)', 'x_2(t)');
saveplot(gcf, [savedir 'swlin_dwelltime6_lifted'], 'latex');


%% constrained variant -- run OPdelta, first switch can occur anywhere

cfg2.run = 1;
cfg2.reset = x0;
cfg2.datafile = 'swlin_dwelltime6_constrained';
% cfg2.verb=0;
cfg2.enforceinitialdwelltime = 0;
genmpc(cfg2);

%% process and plot
h = genmpc('plottrial datafile=swlin_dwelltime6_constrained');
cost = 0;
for i = 1:length(h.x),
    cost = cost + h.x(:, i)' * mcfg.Q * h.x(:, i); 
end;
cost
figure; 

subplot(211);
stairs(h.t, h.u, '-k'); ylabel('Mode'); ylim([0.9 2.1]);
subplot(212);
plot(h.t, h.x(1, :), '-k'); hold on;
plot(h.t, h.x(2, :), '--k');
xlabel('t [s]'); ylabel('States'); legend('x_1(t)', 'x_2(t)');
saveplot(gcf, [savedir 'swlin_dwelltime6_constrained'], 'latex');


%% constrained variant -- run OPdelta, first switch only after 6 steps

cfg2.run = 1;
cfg2.reset = x0;
cfg2.datafile = 'swlin_dwelltime6_constrained2';
cfg2.enforceinitialdwelltime = 1;
% cfg2.verb=0;
genmpc(cfg2);

%% process and plot
h = genmpc('plottrial datafile=swlin_dwelltime6_constrained2');
cost = 0;
for i = 1:length(h.x),
    cost = cost + h.x(:, i)' * mcfg.Q * h.x(:, i); 
end;
cost
figure; 

subplot(211);
stairs(h.t, h.u, '-k'); ylabel('Mode'); ylim([0.9 2.1]);
subplot(212);
plot(h.t, h.x(1, :), '-k'); hold on;
plot(h.t, h.x(2, :), '--k');
xlabel('t [s]'); ylabel('States'); legend('x_1(t)', 'x_2(t)');
saveplot(gcf, [savedir 'swlin_dwelltime6_constrained2'], 'latex');


%% multistep variant -- run OPD considering 

% cfg1.run = 1;
% sigma0 = 2;
% x = x0; for j = 1:6, x = mcfg.A{sigma0} * x; end;
% cfg1.reset = x;
% cfg1.datafile = 'swlin_dwelltime6_sigma0=2test';
% genmpc(cfg1);
% 
% h = genmpc(['plottrial datafile=' cfg1.datafile]);
% cost = 0; x = x0; for j = 1:6, cost = cost + x' * mcfg.Q * x; x = mcfg.A{sigma0} * x; end;
% if any(x ~= h.x(:, 1)), fprintf('State mismatch %f\n', max(abs(x - h.x(:, 1)))); end;
% for i = 1:length(h.x)-1,
%     x = h.x(:, i);
%     for j = 1:6, cost = cost + x' * mcfg.Q * x; x = mcfg.A{h.u(i)} * x; end;
%     if any(x ~= h.x(:, i+1)), fprintf('State mismatch %g\n', max(abs(x - h.x(:, i+1)))); end;
% end;
% % add also the last cost for completeness
% cost = cost + h.x(:, end)' * mcfg.Q * h.x(:, end); 
% cost
% 
% %% run OPD with dwell time constraint asking just for the first sequence
% 
% model = switchedlin_problem('model', cfg2.model_params{:});
% % call OPD directly and run it in fixed-depth mode, rather than fixed-budget mode
% plancfg.n = [];
% plancfg.depth = 30;             % this is about the limit of computational feasibility in this example
% plancfg.actiontype = 'seq';     % return the whole action sequence
% plancfg.U = cfg.planparams.U;
% plancfg.gamma = cfg.gamma;
% plancfg.verb = 3.5;             % show depths reached to get a feel of how algo is doing
% [useq, xseq, planinfo, plandata] = opddwelltime(x0, model, plancfg);
% % the bounds are found in planinfo.bounds, [lower upper]
% planinfo.bounds
% % but in fact the real gap is:
% realgap = diff(planinfo.bounds)

