function dx = quaddyn2(t, x, m, u)
%  Implements the transition function of the inverted pendulum setup
%
%  Parameters:
%   T           - the current time. Ignored, added for compatibility with
%               the ODEXX function style.
%   X           - the current state
%   M           - the model structure specifying physical params
%   U           - the command
%
%  Returns:
%   DX          - the derivative of the state
%

M = 1.3; % Kilos
g = 9.8; % gravity
L = 0.3050; % Meters
I = 0.0605; % Kilos

dx(1,1) = x(4);
dx(2,1) = x(5);
dx(3,1) = x(6);
dx(4,1) = g*sin(x(3));
dx(5,1) = g*sin(x(3)) - g;
dx(6,1) = (-L*u)/I;