function [curreg]=batch_eval_S(testcfg)

%% Test common config

TESTCFG.problem = 'ip';
TESTCFG.test_type = 'ipoff';
TESTCFG.test_postfix = '';
TESTCFG.algorithms = {};
TESTCFG.nsimarray = [];
TESTCFG.osp_Svals = [1];
TESTCFG.ovsp_alfa = 1;
TESTCFG.N = intmax;

TESTCFG.plotStyle = 'oneplot'; %whether to use subplot or separate plots
TESTCFG.avgRegretGrapth = true;
TESTCFG.showDepthGraph = false;
TESTCFG.regretPoints = 1;

testcfg = checkparams(testcfg, TESTCFG);

test_problem=testcfg.problem;
test_type  = testcfg.test_type;
test_postfix = testcfg.test_postfix;
algorithms = testcfg.algorithms;
nsimarray =  testcfg.nsimarray;
OSP_Svals = testcfg.osp_Svals;
OVSP_alfa = testcfg.ovsp_alfa;
N = testcfg.N;

plotStyle = testcfg.plotStyle; %whether to use subplot or separate plots
avgRegretGrapth = testcfg.avgRegretGrapth;
showDepthGraph = testcfg.showDepthGraph;
regretPoints = testcfg.regretPoints;

%% Test common config
if strcmp(test_type,'onl'),
    test_result_type = 'return';
else
    test_result_type = 'regret';
end;

switch test_problem,
    case 'ip',
        Uval = 2;   Uvalstring = ['_U' num2str(Uval)];    Uvalstring = strrep(Uvalstring,'.','_');
        probl_params = [test_problem test_type Uvalstring test_postfix '_'];
        ccfg = ip_common_config(Uval,'coarse');
    case 'hiv',
        probl_params = [test_problem test_type test_postfix '_'];
        ccfg = hiv_common_config();
end;

%% run eval
plotlinestyle = {
    'b-o', 'r-s', 'g-*', 'c-d', 'm-d', 'b-s', 'g-o', 'r-d', 'c-*', 'm-o'};%,...
%     {'b-o', 'r--*', 'g-*', 'c-d', 'm-d', 'b-s', 'g-o', 'r-d', 'c-*', 'm-o'},...
%     {'b-o', 'r-.d', 'g-*', 'c-d', 'm-d', 'b-s', 'g-o', 'r-d', 'c-*', 'm-o'},...
%     {'b-o', 'r:^', 'g-*', 'c-d', 'm-d', 'b-s', 'g-o', 'r-d', 'c-*', 'm-o'},...
%     {'b-o', 'r-v', 'g-*', 'c-d', 'm-d', 'b-s', 'g-o', 'r-d', 'c-*', 'm-o'}...
%     };
avgreg   = zeros(size(algorithms,2), size(nsimarray,2)); %array for storing average regret/return for each algo, for each 'n' from the testset
reg      = zeros(size(algorithms,2), size(nsimarray,2), 403); 
avgdepth = zeros(size(algorithms,2), size(nsimarray,2));

for S=OSP_Svals,
    for algo_i = 1:size(algorithms,2),
        figure(1);  close(1);
        algo_names{algo_i} = [algorithms{algo_i}{1} algorithms{algo_i}{2}];
        if strcmp(algorithms{algo_i}{2},'_v10'),
            algo_names{algo_i} = [algo_names{algo_i} '_alfa=0.' num2str(OVSP_alfa*10)];
        end
        algo_names{algo_i} = strrep(algo_names{algo_i},'_','\_');
        for nsim=nsimarray,
            algo_name    = algorithms{algo_i}{1};
            algo_version = algorithms{algo_i}{2};
            nsim_i = nsimarray==nsim;
            
            algo_params = ['_nsim=' num2str(nsim)];
            switch algo_name
                case {'osp','usp'}
                    algo_params = [algo_params '_S=' num2str(S)];
                case {'osp_oc','osp_oc_reuse'}
                    algo_params = [algo_params '_S=' num2str(S) '_N=' num2str(N)];                    
                case {'ovsp','ossp'}
                    algo_params = [algo_params '_alfa=' num2str(OVSP_alfa*10)];
                otherwise
            end;
            
            
            ccfg.datafile = [probl_params algo_name algo_version algo_params];
            
            switch test_type,
                case 'onl'
                    figure(1);
                    hist=genmpc(['plottrial addtocrt datafile=' ccfg.datafile]);
                    avgreg(algo_i, nsim_i)=hist.R(1); 
                otherwise
                    if avgRegretGrapth,
                        [avgreg(algo_i, nsim_i)] = genmpc_exp('regret', ccfg);
                    else
                        [avgreg(algo_i, nsim_i) reg(algo_i, nsim_i, :)] = genmpc_exp('regret', ccfg);
                        avgReg = 0;
                         for i=[198+[0:regretPoints-1] 206-[0:regretPoints-1]], avgReg = avgReg + reg(algo_i, nsim_i,i); end
                         avgreg(algo_i, nsim_i) = double(avgReg)/(2*regretPoints);
%                         avgreg(algo_i, nsim_i) = reg(algo_i, nsim_i, 206);
                    end;

                    if showDepthGraph,
                        load(ccfg.datafile);
                        avgK=0; 
                        for i=1:size(planinfoh,2), avgK = avgK + double(planinfoh(i).depth); end
                        avgK = double(avgK)/size(planinfoh,2);
                        avgdepth(algo_i, nsim_i) = avgK; %ccfg.gamma^double(avgK)/(1-ccfg.gamma);        
                    end;
            end;
        end;
    end;
    %plot the results
    switch plotStyle
        case 'subplots'
            figure(2);     
            hold off;
    %         subplot((size(OVSP_alfa,2)>1) + 1,ceil(size(OVSP_alfa,2)/2),find(alfa==OVSP_alfa));
            subplot((size(OSP_Svals,2)>1) + 1,ceil(size(OSP_Svals,2)/2),find(S==OSP_Svals));
        case 'oneplot'
            figure(2);     
            hold on;
        case 'separate'
            figure;
    end
    for algo_i = 1:size(algorithms,2),
        hold on;
        if ~strcmp(algorithms{algo_i}{2},'_v10'),
            plot(nsimarray,avgreg(algo_i,:),plotlinestyle{algo_i});
        end;
    end;
    if ~strcmp(plotStyle,'subplots'), legend(algo_names); end;
    title([test_problem ' onestate ' test_result_type ', S=' num2str(S) ', alfa=' num2str(OVSP_alfa)]); 

    if showDepthGraph,
        switch plotStyle
        case 'subplots'
            figure(3); 
            subplot((size(OSP_Svals,2)>1) + 1,ceil(size(OSP_Svals,2)/2),find(S==OSP_Svals));
        case 'oneplot'
            figure(3);
            hold on;
        case 'separate'
            figure;
        end
        for algo_i = 1:size(algorithms,2),
            hold on;
            plot(nsimarray,avgdepth(algo_i,:),plotlinestyle{algo_i});        
        end;
        if strcmp(plotStyle,'subplots'), legend(algo_names); end;
        title([test_problem ' onestate treedepth, S=' num2str(S)]); 
    end;    
%     pause;
end;

 switch plotStyle
    case 'subplots'
        figure(2);
        legend(algo_names);
        if showDepthGraph,
            figure(3);
            legend(algo_names);
        end
    case 'oneplot'
        figure(2);
        legend(OSP_Svals);
        if showDepthGraph,
            figure(3);
            legend(OSP_Svals);
        end
    case 'separate'
 end
        
curreg = avgreg(algo_i,:);