%% IP Common config
close all;  clear all;
Uval = 2;
Uvals = '_U2';

ccfg = ip_common_config(Uval,'coarse');
[approx, theta] = fuzzyqi(['approx datafile=ip_fzqi_nearopt' Uvals]);
ccfg.hstar = @(x) approx.h(approx, theta, x);
ccfg.Qstar = @(x, u) approx.q(approx, theta, x, u); 

mfilename = ['ipoff' Uvals]; % Ts=0.025 maxstep=160 gamma=0.98

%% OPD
cfg = ccfg;
cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 300;         % budget of expansions at each step
cfg.datafile = 'ip_opd';
cfg.datafile = ['ip_opd_U' num2str(Uval) '_n=' num2str(cfg.planparams.n)];
% use visualization -- set visualize to 0 to disable
% cfg.saveplandata = 1;
cfg.visualize = 4;             % this means: only visualize system steps

% run the algorithm
cfg.run = 1;
genmpc(cfg);              

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));


%% ONLINE OSP with n
ccfg.maxsteps = 160;
cfg = ccfg;
cfg.planalg = 'ossp3';            % variant: optimistic planning for deterministic systems
% cfg.planparams.nsim = 6000;         % budget of expansions at each step
cfg.planparams.n = 2500/4;         % budget of expansions at each step
cfg.planparams.S = 1;
cfg.planparams.alfa=0.7;
cfg.datafile = ['iponl_c_' cfg.planalg];
cfg.datafile = strrep(cfg.datafile,'.','_');
cfg.visualize = 4;
% cfg.saveplandata = 1;

% run the algorithm
cfg.run = 1;
genmpc(cfg);          

genmpc(['ploth datafile=' cfg.datafile]);

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));

%% ONLINE OSP with nsim
cfg = ccfg;
cfg.maxsteps = 50;
cfg.planalg = 'ovsp_c1_v6S';            % variant: optimistic planning for deterministic systems
% cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
cfg.planparams.nsim = 6000;         % budget of expansions at each step
% cfg.planparams.n = 2000;         % budget of expansions at each step
cfg.planparams.S = 1;
cfg.datafile = ['iponl_U' num2str(Uval) '_' cfg.planalg '_nsim=' num2str(cfg.planparams.nsim) '_S=' num2str(cfg.planparams.S)];
cfg.visualize = 4;
% cfg.saveplandata = 1;

% run the algorithm
cfg.run = 1;
genmpc(cfg);          

genmpc(['ploth datafile=' cfg.datafile]);

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));


%% OFFLINE OSP with nsim
cfg = ccfg;
cfg.maxsteps = 50;
% cfg.planalg = 'ovsp_c1_v6S';            % variant: optimistic planning for deterministic systems
cfg.planalg = 'osp';            % variant: optimistic planning for deterministic systems
% cfg.planparams.nsim = 6000;         % budget of expansions at each step
cfg.planparams.n = 2000;         % budget of expansions at each step
% cfg.planparams.S = 1;
cfg.datafile = ['iponl_U' num2str(Uval) '_' cfg.planalg '_n=' num2str(cfg.planparams.n)];
% cfg.visualize = 4;
cfg.saveplandata = 1;
cfg.online = 0;
% run the algorithm
cfg.run = 1;
genmpc(cfg);          

% genmpc(['ploth datafile=' cfg.datafile]);
% figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
% fprintf('total return=%.5f\n\n',hist.R(1));