function [xplus, rplus, terminal] = rotarypend_mdp(m, x, u)
% Does not support: noise, refmodel on input; pdf on output

% saturate input
u = max(-m.maxu, min(m.maxu, u));

if m.odemethod(1) == 'v',          % Variable-step ODE
	[odet, odey] = m.odesolver(rotarypend_trans, m.odet, x, m.odeopt, m, u);
    xplus = odey(end, :)';
elseif m.odemethod(1) == 'f',      % Fixed-step ODE
    odey = m.odesolver(rotarypend_trans, m.odet, x, m, u);
    xplus = odey(end, :)';
elseif m.odemethod(1) == 'e',      % Euler, x(k+1) = x(k) + Ts * f(x(k), u(k)) (possibly multistep)
    dt = m.Ts / m.odesteps; xplus = x;
    for i = 1:m.odesteps, xplus = xplus + dt * rotarypend_trans(0, xplus, m, u); end;
end;

% wrap angles and saturate angular velocities
xplus = [mod(xplus(1) + pi, 2*pi) - pi; max(-m.maxx(2), min(m.maxx(2), xplus(2))); ...
    mod(xplus(3) + pi, 2*pi) - pi; max(-m.maxx(4), min(m.maxx(4), xplus(4)))];

% compute reward
if m.rewtype(1) == 'n',
    rplus = 1 - (x' * m.Qrew * x + m.Rrew * u^2) / m.nonnorm_maxr;
end;

% task is continuing
terminal = 0;

end     % rasetup_singletransition local function


% Transition function of the inverted pendulum setup
function dx = rotarypend_trans(t, x, m, u)
% pick up variables
% theta = x(1);
thetadot = x(2);
alpha = x(3);
alphadot = x(4);

frac = 1/(m.a * m.c - m.b^2 * cos(alpha)^2);

thetadotdot = frac * (-m.b * m.c * sin(alpha) * alphadot^2 + m.b * m.d * sin(alpha) * cos(alpha) ...
    - m.c * m.e * thetadot + m.c * m.f * u);

alphadotdot = frac * (m.a * m.d * sin(alpha) - m.b^2 * sin(alpha) * cos(alpha) * alphadot^2 ...
    - m.b * m.e * cos(alpha) * thetadot + m.b * m.f * cos(alpha) * u);

dx = [thetadot; thetadotdot; alphadot; alphadotdot];
end