function out = int_problem(what, varargin)
% Integrator problem setup.
%   OUT = DOUBLEINT_PROBLEM(WHAT, [CFG])
% This function conforms to the specifications established by SAMPLE_PROBLEM.
% For now, CFG can only be a reward type: lqr, nlqr, mintime

maxx = 5;
maxu = 2;
gridstep = .5;
xgrids = {-maxx:gridstep:maxx};
ugrids = {-maxu:gridstep:maxu};
x0 = (2*rand) * maxx - maxx;
gamma = 0.98;
switch what

    case 'model'
        model.id = 'int'; model.problem = @int_problem;
        model.det = 1;
        model.p = 1;
        model.q = 1;
        
        model.maxx = maxx;
        model.maxu = maxu;
        model.K = 2;       % command gain
        
        % reward specification
        if nargin >= 2,
            model.rewtype = varargin{1};
        else
            model.rewtype = 'lqr';
        end;
        switch model.rewtype,
            case 'lqr',
                model.Q = .1; model.R = .05;
            case 'nlqr',
                model.Q = .1; model.R = .05;
                model.nonnorm_maxr = model.Q * maxx^2 + model.R * maxu^2;
                model.maxr = 1;
             case 'mintime',
               model.zeroband = 0.1;
               model.zeroreward = 10;
        end;

        model.Ts = 1;
        model.fun = @int_mdp;
        out = model;

    case 'fuzzy'
        cfg.xgrids = xgrids;
        cfg.ugrids = ugrids;
        cfg.gamma = gamma;
        cfg.x0 = x0;
    
        out = cfg;

    case 'reset',   % handle state reset
        RES.type = [];
        RES.x0 = [];
        model = varargin{1};
        res = varargin{2};
        if ischar(res) && any(strcmp(res, {'rand'})),   % handle simple method identifier
            res = struct('type', res);
        elseif isnumeric(res),
            res = struct('type', 'values', 'x0', res);
        end;
        
        res = parseconfig(res, RES);
        switch res.type,
            case 'rand',
                x0 = (2*rand(model.p, 1) - 1) .* model.maxx;
            case 'values',
                if size(res.x0, 2) > 1,    x0 = res.x0(:, unidrnd(size(res.x0, 2)));
                else                        x0 = res.x0;
                end;
        end;                    
        out = x0;

end;


% END doubleint_problem(), RETURNING out ====================================