X0 = [0.1534 2*0.1534 -0.1561 0]';

cfg.mc_maxsteps = 300;

model = sw_problem('model');

% time
Ts = model.Ts; 
Tend = cfg.mc_maxsteps * Ts;
T = 0:Ts:Tend;
% ode settings
odeopt  = odeset('events', @sw_events, 'MaxStep', Ts);
% initialize simulation for i-th initial state 
X = nan(cfg.mc_maxsteps+1, model.p);
global SWMCRBFNEAREST_U
SWMCRBFNEAREST_U = zeros(cfg.mc_maxsteps+1, 1);
% simulate
timespan = T;
kstart = 1; koffset = 0;
[t,x,te,ye,ie] = ode45(@sw_dynamics, timespan, X0, odeopt, model, [], [], []); 
tend = t(end);
kend = floor(tend/Ts)+1;
% resume simulation only as long as the timespan is not exhausted,
% and the detected event was a swing leg impact (index 1) and not failure (index 2)
while tend < Tend  &&  ie(1) == 1,
    X(kstart:kend, :) = x((kstart:kend) - koffset, :);
    % there was an impact, compute discontinuous state transfer
    x_afterimpact = sw_impact(x(end,:));
    % continue simulation
    firstsample = ceil(tend/Ts) * Ts;
    kstart = kend + 1; 
    if tend < firstsample,
        timespan = [tend firstsample:Ts:Tend];
        koffset = kstart - 2;
    else
        timespan = firstsample:Ts:Tend;
        koffset = kstart - 1;
    end;
    [t,x,te,ye,ie] = ode45(@sw_dynamics, timespan, x_afterimpact, odeopt, model, c, rad, h);
    tend = t(end);
    kend = floor(tend/Ts)+1;
end;
X(kstart:kend, :) = x((kstart:kend) - koffset, :);
% pad with failure state at the next sample time,
% if simulation ended with failure
if ~isempty(ie) && ie(1) == 2,
    Kend = kend+1;
    X(kend+1, :) = x(end, :);
    fail = 1;
else
    Kend = kend;
    fail = 0;
end;

hist.x = X(1:Kend, :)';
hist.t = 0:Ts:(Kend-1)*Ts;
figure(10); clf;
subplot(211); hold on; plot(hist.t, hist.x(1, :), 'k', hist.t, hist.x(3, :), 'b'); legend('\theta', '\theta''');
subplot(212); hold on; plot(hist.t, hist.x(2, :), 'k', hist.t, hist.x(4, :), 'b'); legend('\phi', '\phi''');


[T101, X101] = ode45_101('sw_101', 0, tend, X0);
X101 = X101';
figure(11); clf;
subplot(211); hold on; plot(T101, X101(1, :), 'k', T101, X101(3, :), 'b'); legend('\theta', '\theta''');
subplot(212); hold on; plot(T101, X101(2, :), 'k', T101, X101(4, :), 'b'); legend('\phi', '\phi''');
