function figh = sw_animate(hist, cfg)
% Displays an animation of the walker motion
%   SW_ANIMATE(HIST, CFG)
% Parameters:
%   HIST    - the evolution history, as e.g., outputted by fuzzyqi('replay')
%   CFG     - (optional) configuration, see commented defaults below

if nargin < 2, cfg = struct(); end;
% Config defaults
CFG.len = 1;                % length of leg
CFG.stancecolor = 'b';      % color of stance leg
CFG.swingcolor = 'r';       % color of swing leg
CFG.figsize = [];           % initial size of the figure (will get reshaped by axis equal image)
CFG.figname = 'SimpleWalker Motion';        % name of figure
CFG.pausebeforerun = 1;     % whether to pause after initial display
% process config
cfg = checkparams(cfg, CFG);

len = cfg.len;
% create figure
figargs = {};
if ~isempty(cfg.figsize), figargs = {figargs{:}, 'Position', [0 0 cfg.figsize]}; end;
if ~isempty(cfg.figname), figargs = {figargs{:}, 'Name', cfg.figname}; end;
    
figh = figure(figargs{:}); movegui(figh, 'center');

% create figure
axis(gca, [-1.1*len 1.1*len -1.1*len .2*len]);
axis image; 
axis off;

% create legs
legs = [0 0];
legs(1) = line([0 0], [0, -len], 'Color', cfg.stancecolor, 'LineWidth', 5);
legs(2) = line([0 0], [0, -len], 'Color', cfg.swingcolor, 'LineWidth', 5);

for k = 1:size(hist.x, 2)
    theta = hist.x(1, k);
    phi = hist.x(2, k);
    set(legs(1), 'XData', [0 len*sin(theta)], 'YData', [0 -len*cos(theta)]);
    set(legs(2), 'XData', [0 -len*sin(phi-theta)], 'YData', [0 -len*cos(phi-theta)]);
    drawnow;
    if (k == 1) && (cfg.pausebeforerun) pause; end;
end;

% pause; close(figh);

% END sw_animate RETURNING void ============================================================
