%% 0) Initialization: Cleanup & problem settings
% Let's first configure the problem settings. We will use two problems, the DC motor and the robot
% arm. Note the extensive use of configuration structures.

clear all; close all; clc;  % cleanup 
hccfg = struct;
hccfg.target='screen';
hccfg.problem = 'hillcar_problem';
hcfg.model_params={'maxx=[1;3]','maxu=1','odemethod=fixedode odesolver=@ode4 Ts=0.1'};
hccfg.gamma = .98;        % discount factor 

%% 1) FQI Hill Car Problem

% remove the data file if it exists
if exist('hillcar_fittedqidemo.mat', 'file'), delete('hillcar_fittedqidemo.mat'); end;

cfg = hccfg;                        % start from the problem config
cfg.run = 1;                        % set the "run" flag
cfg.term = 'zero';                  % zero Q-values of terminal states
cfg.datadir = pwd;                  % save the data in the current directory
cfg.datafile = 'hillcar_fittedqidemo';   % in this file
cfg.maxiter = 50;                  % run for this number of iterations (no convergence test by default)
cfg.U = {[-1 0 1]};               % three discrete actions
cfg.samples = 'method=randdisc N=6000';
    % use 3000 samples, uniformly distributed throughout the state-discrete action space                   
cfg.regmethod = 'extratrees';       % use ensembles of extratrees to approximate
cfg.singlereg = 1;                  % use a single tree for all the discrete actions (usually works better)
cfg.trees_ntrees = 25;              % # of trees in the ensemble
cfg.trees_nmin = 2;                 % nmin param for extra trees
% Call the algorithm
fittedqi(cfg);
 

%% Examine the Q-function and policy
hillcar_plot('fittedqiq datafile=hillcar_fittedqidemo slice=[NaN,NaN,1]');
hillcar_plot('fittedqih datafile=hillcar_fittedqidemo');