% A script to test llrfittedqi

%% Common config
ccfg = struct;
ccfg.problem = 'ipsetup_problem';
ccfg.gamma = .98;
ccfg.maxiter  = 300;
ccfg.datadir = 'd:\work\m\approxrl\testing\test_llrfittedqi_llrapprox_ipsetup\';
ccfg.knn = 10;
ccfg.samples = 'method=randdisc N=10000';
ccfg.U = {[-3 0 3]};
ccfg.verb = 2;


%% Trajectory sampling test 1 -- random policy
cfg = ccfg;
cfg.samples.method = 'trajdisc';
cfg.samples.ugrids = cfg.U;
cfg.samples.N = 3200;
cfg.samples.traj_K = 400;
cfg.samples.traj_X0 = [-pi, 0]';
cfg.samples.policy = @hrand;
cfg.samples.policyargs = {flat(cfg.U), 1};    % 1 = discrete output
cfg.datafile = 'test_llrfittedqi_trajdisc';
cfg.run = 1;
llrfittedqi(cfg);

%% Trajectory sampling test 1 -- random policy
% get a reasonable solution to produce informative samples
load d:\work\m\approxrl\systems\invpend_setup\llrfqi_ip_lspicomparison\lspi_ip_Nsgrid_triang_N11x11_grid_xgr71x71 approx theta

cfg = ccfg;
cfg.samples.method = 'trajdisc';
cfg.samples.ugrids = cfg.U;
cfg.samples.N = 3200;
cfg.samples.traj_K = 400;
cfg.samples.traj_X0 = [-pi, 0]';
cfg.samples.policy = @(x, t) approx.h(approx, theta, x);
cfg.samples.explor = .2;                        % 10% exploration prob
cfg.datafile = 'test_llrfittedqi_trajdisc_info';
cfg.run = 1;
llrfittedqi(cfg);



%% Run using kdtrees
cfg = ccfg;
cfg.datafile = 'test_llrfittedqi_kdtree';
cfg.run = 1;
cfg.knnmethod = 'kdtree';
llrfittedqi(cfg);


%% Run using Matlab JIT function, with the same samples
% CHECKS OUT -- same results
cfg = ccfg;
cfg.datafile = 'test_llrfittedqi_jit';
cfg.loadsamples = 'test_llrfittedqi_kdtree';    % make sure we use same samples => results should be identical
cfg.run = 1;
cfg.knnmethod = 'jit';
llrfittedqi(cfg);


%% Run using updated function (local weighting), same samples; JIT code
% Use same samples and uniform weighting, should give same results
% CHECKS OUT -- same results
cfg = ccfg;
cfg.datafile = 'test_llrfittedqi_weighted';
cfg.loadsamples = 'test_llrfittedqi_kdtree';    % make sure we use same samples => results should be identical
cfg.run = 1;
cfg.wfun = 'unif';
cfg.knnmethod = 'jit';
llrfittedqi(cfg);

%% Run using updated function (local weighting), same samples; kdtree
% Use same samples and uniform weighting, should give same results
% CHECKS OUT -- same results
cfg = ccfg;
cfg.datafile = 'test_llrfittedqi_weighted_kdtree';
cfg.loadsamples = 'test_llrfittedqi_kdtree';    % make sure we use same samples => results should be identical
cfg.run = 1;
cfg.wfun = 'unif';
cfg.knnmethod = 'kdtree';
llrfittedqi(cfg);


%% Run using scaled distance metric (and Matlab JIT function)
cfg = ccfg;
cfg.datafile = 'test_llrfittedqi_S';
cfg.loadsamples = 'test_llrfittedqi_jit';
cfg.run = 1;
cfg.knnmethod = 'jit';
cfg.S = 1 ./ ([pi 15*pi].^2);           % weigh distance according to state space dimensions
llrfittedqi(cfg);


%% below experiments no longer used
% %% Run using smaller neighborhood
% cfg = ccfg;
% cfg.datafile = 'test_llrfittedqi_k5';
% cfg.knn = 5;
% cfg.run = 1;
% cfg.knnmethod = 'jit';
% cfg.S = 1 ./ ([pi 15*pi]);           % weigh distance according to state space dimensions
% llrfittedqi(cfg);
% 
% 
% %% Run using more samples
% cfg = ccfg;
% cfg.datafile = 'test_llrfittedqi_Ns20000';
% cfg.samples = 'method=randdisc N=20000';
% cfg.run = 1;
% cfg.knnmethod = 'jit';
% cfg.S = 1 ./ ([pi 15*pi]);           % weigh distance according to state space dimensions
% llrfittedqi(cfg);
% 
% 
% %% Run using grid of samples
% 
% cfg = ccfg;
% cfg.datafile = 'test_llrfittedqi_gridN80M3';
% cfg.samples = 'method=grid xgrids=[80,80] ugrids=3';
% cfg.run = 1;
% cfg.knnmethod = 'jit';
% cfg.S = 1 ./ ([pi 15*pi]);           % weigh distance according to state space dimensions
% llrfittedqi(cfg);
% 
% 
