
%% model config
mcfg = struct;
mcfg.type = 'doubleint';
mcfg.nagents = 5;
mcfg.dim = 2;
mcfg.range = 5;
%mcfg.flock_dist = 3;
mcfg.flock_maxdvel = 5;
mcfg.Ts = 1;

%% algo config
cfg = struct;
cfg.problem = 'malinear_problem';
cfg.model_params = {mcfg};
cfg.planalg = 'masimplecontrol';
cfg.planparams.type = 'flock';
% cfg.planparams.alpha = .1;
cfg.visualize = 2;
cfg.viscfg.pause = .01;
cfg.datadir = 'pwd';
cfg.reset = 'rand';

% run it
cfg.run = 1;
magenmpc(cfg);


%% test reward function for 1dim, two agents

clear all; close all;
mcfg = struct;
mcfg.type = 'doubleint';
mcfg.nagents = 2;
mcfg.dim = 1;
mcfg.range = 5;
mcfg.flock_dist = 2;
mcfg.flock_maxdvel = 10;
mcfg.Ts = .25;
m = malinear_problem('model', mcfg);

p1 = 10;
v1 = 0;
p2 = p1 + (-30:.5:30);
v2 = -30:.5:30;
X = flat({p1, v1, p2, v2});
mcfg

%%
Xplus = zeros(sum(m.p), length(X));
Rplus = zeros(m.nagents, length(X));
Nplus = zeros(m.nagents, m.nagents, length(X));
for i = 1:length(X),
    [Xplus(:, i), Rplus(:, i), term, Nplus(:, :, i)] = malinear_mg(m, X(:, i), [0; 0], 'real');
end;
subplot(2, 1, 1);
mesh(p2, v2, reshape(Rplus(1, :), length(p2), length(v2))');

subplot(2, 1, 2);
mesh(p2, v2, reshape(Rplus(2, :), length(p2), length(v2))');

%%
Xplus = zeros(sum(m.p), length(X));
Rplus = zeros(1, length(X));
Nplus = zeros(m.nagents, length(X));
for i = 1:length(X),
    [Xplus(:, i), Rplus(i), term, Nplus(:, i)] = malinear_mg(m, X(:, i), [0; 0], 'pred', 1, [0 1]);
end;

subplot(2, 1, 1);
mesh(p2, v2, reshape(Rplus(1, :), length(p2), length(v2))');

Xplus = zeros(sum(m.p), length(X));
Rplus = zeros(1, length(X));
Nplus = zeros(m.nagents, length(X));
for i = 1:length(X),
    [Xplus(:, i), Rplus(i), term, Nplus(:, i)] = malinear_mg(m, X([3 4 1 2], i), [0; 0], 'pred', 2, [1 0]);
end;
subplot(2, 1, 2);
mesh(p2, v2, reshape(Rplus(1, :), length(p2), length(v2))');


%% test reward function for 2dim, two agents

mcfg.type = 'doubleint';
mcfg.nagents = 2;
mcfg.dim = 2;
mcfg.range = 5;
mcfg.flock_dist = 3;
mcfg.flock_maxdvel = 10;
mcfg.Ts = .25;
m = malinear_problem('model', mcfg);

p21 = (-10:.5:10);
p22 = (-10:.5:10);
X = flat({0, 0, 0, 0, p21, p22, 0, 0});

Xplus = zeros(sum(m.p), length(X));
Rplus = zeros(1, length(X));
Nplus = zeros(m.nagents, length(X));
for i = 1:length(X),
    [Xplus(:, i), Rplus(i), term, Nplus(:, i)] = malinear_mg(m, X(:, i), zeros(4, 1), 'pred', 1, [0 1]);
end;

mesh(p21, p22, reshape(Rplus(1, :), length(p21), [])');

%% test reward function for 1dim, 2 agents

mcfg.type = 'doubleint';
mcfg.nagents = 3;
mcfg.dim = 1;
mcfg.range = 5;
mcfg.flock_dist = 3;
mcfg.flock_maxdvel = 10;
mcfg.Ts = .25;
m = malinear_problem('model', mcfg);

p1 = (-20:.5:20);
p2 = (-20:.5:20);
X = flat({0, 0, p1, 0, p2, 0});

Xplus = zeros(sum(m.p), length(X));
Rplus = zeros(1, length(X));
Nplus = zeros(m.nagents, length(X));
for i = 1:length(X),
    [Xplus(:, i), Rplus(i), term, Nplus(:, i)] = malinear_mg(m, X(:, i), zeros(3, 1), 'pred', 1, [0 1 1]);
end;

mesh(p1, p2, reshape(Rplus(1, :), length(p1), [])');



