% Test enhancements in OPD (tree and memory reuse), comparing to OPSS applied to det as
% baseline

%% OPD baseline
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opd';
cfg.planparams.n = 100;
cfg.planparams.U = [-3 0 3];
cfg.planparams.verb = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 50;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_opd';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_opd');

%% Tree reuse: OPSS baseline
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opss';
cfg.planparams.n = 100;
cfg.planparams.L = 1;
cfg.planparams.U = [-3 0 3];
cfg.planparams.verb = 1;
cfg.planparams.reusetree = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 40;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_opd_treereuse_opssbaseline';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_opd_treereuse_opssbaseline');


%% Tree reuse: OPD 
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opd';
cfg.planparams.n = 100;
cfg.planparams.U = [-3 0 3];
cfg.planparams.verb = 1;
cfg.planparams.reusetree = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 40;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_opd_treereuse';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_opd_treereuse');



%% Memory reuse: OPSS baseline
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opss';
cfg.planparams.n = 100;
cfg.planparams.L = 1;
cfg.planparams.U = [-3 0 3];
cfg.planparams.verb = 1;
cfg.planparams.usemem = 1; 
cfg.planparams.lipschitz = 0.25;  
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 40;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_opd_usemem_opssbaseline';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_opd_usemem_opssbaseline');


%% Memory reuse: OPD 
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opd';
cfg.planparams.n = 100;
cfg.planparams.U = [-3 0 3];
cfg.planparams.verb = 1;
cfg.planparams.usemem = 1; 
cfg.planparams.lipschitz = 0.25;  
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 40;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_opd_usemem';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_opd_usemem');



%% Test memory with OPD in simple integrator with visualization
cfg = struct;

cfg.problem = 'int_problem';
cfg.model_params = {'nlqr'};
cfg.gamma = 0.9;
cfg.planalg = 'opd';
cfg.planparams.n = 100;
cfg.planparams.L = 1;
cfg.planparams.U = [-2 2];
cfg.planparams.usemem = 1;
cfg.planparams.lipschitz = 1;  
cfg.planparams.verb = 3;        % 3 = visualize memory
%
cfg.online = 1;
cfg.x0 = [-5; 3];
cfg.trials = 1;
cfg.maxsteps = 10;
%
cfg.saveplandata = 0;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_int_opd_usemem';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_int_opd_usemem');


