%% Baseline stoch IP
rand('twister', 10);
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.7,.3] stoch_gain=[1,.7]', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opss';
cfg.planparams.n = 200;
cfg.planparams.L = 2;
cfg.planparams.U = [-3 0 3];
cfg.planparams.verb = 1;
%
cfg.online = 1;
cfg.x0 = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 50;
%
cfg.saveplandata = 0;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_stoch_opss';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_stoch_opss_noreuse'); 


%% reimplementation with b(x) only in opmdp
% for the same random seed, should give exactly the same results as the above (including tree depth)
% rand('twister', 10);
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.7,.3] stoch_gain=[1,.7]', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opmdp';
cfg.planparams.n = 100;
cfg.planparams.L = 2;
cfg.planparams.U = [-3 0 3];%
cfg.online = 1;
cfg.x0 = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 50;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_stoch_opmdp';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_stoch_opmdp'); 


%% opmdp with lipschitz learning
% for the same random seed, should give exactly the same results as the above (including tree depth)
% rand('twister', 10);
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.7,.3] stoch_gain=[1,.7]', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opmdp';
cfg.planparams.n = 50;
cfg.planparams.L = 2;
cfg.planparams.U = [-3 0 3];%
cfg.planparams.blearn = @learn_lipschitz;
cfg.planparams.lipschitz = 0.5;
cfg.planparams.mem_maxsize = 300;
cfg.planparams.verb = 3;
cfg.online = 1;
cfg.x0 = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 50;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_stoch_opmdp_lipschitzlearn';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_stoch_opmdp_lipschitzlearn'); 



%% opmdp with local lssvr learning
% for the same random seed, should give exactly the same results as the above (including tree depth)
% rand('twister', 10);
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.7,.3] stoch_gain=[1,.7]', 'Ts=0.05'};
cfg.gamma = 0.95;
cfg.planalg = 'opmdp';
cfg.planparams.n = 50;
cfg.planparams.L = 2;
cfg.planparams.U = [-3 0 3];%
cfg.planparams.blearn = @learn_llssvr;
cfg.planparams.Knn = 7;
cfg.planparams.Snn = [1;1];
cfg.planparams.svrgamma = 900;
cfg.planparams.svrsigma = 0.2;
cfg.planparams.verb = 3;
cfg.online = 1;
cfg.x0 = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 50;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_ip_stoch_opmdp_llssvr';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_ip_stoch_opmdp_llssvr'); 


%% opmdp with lipschitz learning, on the res arm
cfg = struct;

cfg.problem = 'resonatingarm_problem';
cfg.model_params = {'rewtype=exp stoch_prob=[.5,.5] stoch_gain=[1,.5]'};
cfg.gamma = 0.95;
cfg.planalg = 'opmdp';
cfg.planparams.n = 5;
cfg.planparams.L = 2;
cfg.planparams.U = [-2 0 2];%
% cfg.planparams.blearn = @learn_lipschitz;
% cfg.planparams.lipschitz = 1;
% cfg.planparams.mem_maxsize = 500;
cfg.planparams.verb = 3;
cfg.online = 1;
cfg.reset = [-0.85, 0];
cfg.trials = 1;
cfg.maxsteps = 100;
%
cfg.saveplandata = 1;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'test_resarm_stoch_opmdp_lipschitzlearn';
cfg.run = 1;
genmpc(cfg);

genmpc('plottrial datafile=test_resarm_stoch_opmdp_lipschitzlearn'); 
