
%% SOOP on invpend
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.9;
cfg.model_params = {'rewtype=nlqr Qrew=[1,0;0,0] Rrew=0', 'type=ip maxu=3', 'Ts=0.05'};
cfg.planalg = 'sooplp';
cfg.planparams.n = 500;
cfg.planparams.method = 'soop';
cfg.planparams.dimselect = 'disc';
cfg.planparams.alpha = cfg.gamma; %0.7; % theory indicates we should set this
cfg.planparams.verb = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 70;
%
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'ip_soop';
cfg.run = 1;
genmpc(cfg);

hsoop = genmpc('plottrial datafile=ip_soop');


%% OPC with explicit bound on invpend
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.7;
cfg.model_params = {'rewtype=nlqr Qrew=[1,0;0,0] Rrew=0', 'type=ip maxu=3', 'Ts=0.05'};
cfg.planalg = 'sooplp';
cfg.planparams.n = 1000;
cfg.planparams.method = 'lp';
cfg.planparams.dimselect = 'disc';
cfg.planparams.lipschitz = [1 1];
cfg.planparams.alpha = cfg.gamma;
cfg.planparams.verb = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 70;
%
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'ip_opc';
cfg.run = 1;
genmpc(cfg);

hopc = genmpc('plottrial datafile=ip_opc');


%% OPC with theoretical bound
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.7;
cfg.model_params = {'rewtype=nlqr Qrew=[1,0;0,0] Rrew=0', 'type=ip maxu=3', 'Ts=0.05'};
cfg.planalg = 'sooplp';
cfg.planparams.n = 500;
cfg.planparams.method = 'opctheor';
cfg.planparams.dimselect = 'disc';
cfg.planparams.aconst = 4;
cfg.planparams.alpha = cfg.gamma;
cfg.planparams.verb = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 70;
%
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'ip_opctheor';
cfg.run = 1;
genmpc(cfg);

hopc = genmpc('plottrial datafile=ip_opctheor');

%% SOOP2
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.7;
cfg.model_params = {'rewtype=nlqr Qrew=[1,0;0,0] Rrew=0', 'type=ip maxu=3', 'Ts=0.05'};
cfg.planalg = 'sooplp';
cfg.planparams.n = 1000;
cfg.planparams.method = 'SOOP2';
cfg.planparams.dimselect = 'disc';
cfg.planparams.alpha = cfg.gamma;
cfg.planparams.verb = 1;
% cfg.planparams.removeduplicates = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 70;
%
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'ip_SOOP2';
cfg.run = 1;
% cfg.saveplandata = 1;
genmpc(cfg);

hsoop2 = genmpc('plottrial datafile=ip_SOOP2');

%% SOOP2 - remove duplicates or not
cfg = struct;

cfg.problem = 'ipsetup_problem';
cfg.gamma = 0.9;
cfg.model_params = {'rewtype=nlqr Qrew=[1,0;0,0] Rrew=0.1', 'type=ip maxu=3', 'Ts=0.05'};
cfg.planalg = 'sooplp';
cfg.planparams.n = 3000;
cfg.planparams.method = 'SOOP2';
cfg.planparams.dimselect = 'disc';
cfg.planparams.alpha = cfg.gamma;
cfg.planparams.verb = 1;
cfg.planparams.removeduplicates = 1;
%
cfg.online = 1;
cfg.reset = [-pi; 0];
cfg.trials = 1;
cfg.maxsteps = 70;
%
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'ip_SOOP2_removedup';
cfg.run = 1;
% cfg.saveplandata = 1;
genmpc(cfg);

hsoop2 = genmpc('plottrial datafile=ip_SOOP2_removedup');


%% to check for differences between old and new implem of set initialization
load ip_SOOP2_newinit plandatah; new = plandatah;
load ip_SOOP2_oldinit plandatah; old = plandatah;
th = 1e-10;
for i = 1:length(new),
    dx = max(abs(new{i}.SX(:) - old{i}.SX(:)));
    dr = max(abs(new{i}.SR(:) - old{i}.SR(:)));
    du = max(abs(new{i}.SU(:) - old{i}.SU(:)));
    ds = max(abs(new{i}.SS(:) - old{i}.SS(:)));
    if dx > th || dr > th || du > th || ds > th,
        i
        dx
        dr
        du
        ds
        pause
    end;
end;

%% ROT PEND PROBLEM STARTS HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% OPC on the rotary pendulum problem
cfg = struct;

cfg.problem = 'rotarypend_problem';
cfg.gamma = 0.9;
mcfg.Qrew = diag([0.05 0 1 0]); mcfg.Rrew = 0;
cfg.model_params = {mcfg};
cfg.planalg = 'sooplp';
cfg.planparams.n = 3000;
cfg.planparams.method = 'lp';
cfg.planparams.lipschitz = [1 2];
cfg.planparams.dimselect = 'disc';
cfg.planparams.alpha = cfg.gamma;
cfg.planparams.verb = 1;
%
cfg.online = 1;
cfg.reset = [0; 0; -pi; 0];
cfg.trials = 1;
cfg.maxsteps = 50;
%
cfg.verb = 3;
cfg.datadir = pwd;
cfg.datafile = 'rotpend_opc';
cfg.run = 1;
genmpc(cfg);

hsoop = genmpc('plottrial datafile=rotpend_opc');

