%% General config
% clear all; close all;

% batch experiment
BCFG = struct;
BCFG.alg = 'genmpc';
BCFG.verb = 5;
BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'problem', 'planalg', 'X', 'planparamsn', 'planparamsCp'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

% algorithm
ACFG = struct;
ACFG.problem = 'ipsetup_problem';
ACFG.model_params = {'nlqr', 'type=ip maxu=3 stoch_prob=[.6,.4] stoch_gain=[1,.7]', 'Ts=0.05'};
ACFG.online = 0;
ACFG.gamma = 0.95;
ACFG.planalg = 'uct';
ACFG.planparams.L = 2; % although not directly used
ACFG.planparams.U = [-3 0 3];
% in case we use online
ACFG.trials = 1;
ACFG.maxsteps = 50;
ACFG.reset = [-pi, 0]';
% visualization
ACFG.verb = 3;
ACFG.datadir = 'D:\Work\m\approxrl\testing\test_uct';
ACFG.X = 'coarse';  % results also available for "coarser"


% prepare for possible eval
[approx, theta] = fuzzyqi('approx datafile=D:\Work\m\approxrl\systems\invpend_setup\op_ip\fzqi_ip_nearopt_p04g07_Ts005 verb=-10');
acfg.hstar = @(x) approx.h(approx, theta, x);
acfg.Qstar = @(x, u) approx.q(approx, theta, x, u); 

%% UCT settings

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.id = 'uct';   % note we don't use this as an infix -- it already appears as algorithm name
bcfg.legendprefix = 'UCT';
bcfg.repeat = 1;
bcfg.plotconf = 0;  % the interval is so tight it's useless
% transform input to align it with the OPSS "n"
% bcfg.intransform = @(val) round(val ./ ACFG.planparams.L ./ length(ACFG.planparams.U));

acfg = ACFG;
acfg.planalg = 'uct';
% acfg.planparams.n = [100:200:1000] * ACFG.planparams.L * length(ACFG.planparams.U);
% acfg.planparams.Cp = [0.01 0.1 0.5 1 2];

acfg.planparams.n = 400 * ACFG.planparams.L * length(ACFG.planparams.U);
acfg.planparams.Cp = 1;


%% run
processbatch(bcfg, acfg); 


%% eval across n
bcfg.in = 'planparamsn';
bcfg.out = 'regret';
evalbatch(bcfg, acfg);

%% eval across Cp
bcfg.in = 'planparamsCp';
bcfg.out = 'regret';
evalbatch(bcfg, acfg);

