
%% Main config
mode = 'run';
randseed = 2^20;

% main cfg
ccfg = struct;
ccfg.(mode) = 1;
ccfg.problem = 'ipsetup_problem';
ccfg.gamma = .98;
ccfg.datadir = pwd;
ccfg.approx = 'enhanced type=rbfdisc N=11 M=3';
ccfg.trialtime = 1.5;           % sufficient for stabilization from most (all?) initial states
ccfg.maxtime  = 60*1;           % simulated time
ccfg.explor = 1;
ccfg.explordecay = 0.1^(1/ccfg.maxtime);  
ccfg.dstoretheta = 1;           % store theta once every second
ccfg.verb = 5;
ccfg.ddisp = 3;
ccfg.dsave = 30;                % save once every halfminute
ccfg.verb = 5;
ccfg.eps = 0;

%% LSPEONLINE tests =====================================

lspecfg = ccfg;
lspecfg.dupd = .05;
lspecfg.alpha = .1;
comparevars = {'thetah', 'trun'};

% Results of 1 should ideally be IDENTICAL with 2; 3 IDENTICAL with 4
% 1 and 2 should be approximately equal to 3 and 4

% 1. normal inversion, index not optimized
rand('state', randseed);
cfg = lspecfg;
cfg.method = 'inv';
cfg.indexoptimized = 0;
cfg.datafile = 'test_lspes_inv_dupd005_indexopt0';
% lspeonline(cfg);
loadwithprefix(cfg.datafile, 'in_', comparevars);

% 2. normal inversion, index optimized
rand('state', randseed);
cfg = lspecfg;
cfg.method = 'inv';
cfg.indexoptimized = 1;
cfg.datafile = 'test_lspes_inv_dupd005_indexopt1';
% lspeonline(cfg);
loadwithprefix(cfg.datafile, 'io_', comparevars);

% 3. recursive inversion, index not optimized
rand('state', randseed);
cfg = lspecfg;
cfg.method = 'recinv';
cfg.indexoptimized = 0;
cfg.datafile = 'test_lspes_recinv_dupd005_indexopt0';
% lspeonline(cfg);
loadwithprefix(cfg.datafile, 'rn_', comparevars);

% 4. recursive inversion, index optimized
rand('state', randseed);
cfg = lspecfg;
cfg.method = 'recinv';
cfg.indexoptimized = 1;
cfg.datafile = 'test_lspes_recinv_dupd005_indexopt1';
% lspeonline(cfg);
loadwithprefix(cfg.datafile, 'ro_', comparevars);

for i = 1:30,
    di(i) = norm(in_thetah{i} - io_thetah{i}, 'inf');
    dr(i) = norm(rn_thetah{i} - ro_thetah{i}, 'inf');
    dir(i) = norm(in_thetah{i} - rn_thetah{i}, 'inf');
end;
disp('Diff between opt/nonopt, INV case:');
di
disp('Diff between opt/nonopt, RECINV case:');
dr
disp('Diff between INV and RECINV, nonopt:');
dir
in_trun
io_trun
rn_trun
ro_trun

% LSPE Results: 
%   inv opt/nonopt: identical; 
%   recinv opt/nonopt: 1e-6 errors (numerical?)
%   inv/recinv: 1e-6 errors (expected and acceptable)
% Run time:
%   recinv/opt FASTER THAN inv/opt FASTER THAN inv/nonopt FASTER THAN recinv/nonopt
%   So, best combination is recinv/opt


%% LSPI tests ==========================================

lspicfg = ccfg;
lspicfg.dupd = .05;
comparevars = {'thetah', 'trun'};

% Results of 1 should ideally be IDENTICAL with 2; 3 IDENTICAL with 4
% 1 and 2 should be approximately equal to 3 and 4

% 1. normal inversion, index not optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'inv';
cfg.indexoptimized = 0;
cfg.datafile = 'test_lspis_inv_dupd005_indexopt0';
% lspionline(cfg);
loadwithprefix(cfg.datafile, 'in_', comparevars);

% 2. normal inversion, index optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'inv';
cfg.indexoptimized = 1;
cfg.datafile = 'test_lspis_inv_dupd005_indexopt1';
lspionline(cfg);
loadwithprefix(cfg.datafile, 'io_', comparevars);

% 3. recursive inversion, index not optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'recinv';
cfg.indexoptimized = 0;
cfg.datafile = 'test_lspis_recinv_dupd005_indexopt0';
% lspionline(cfg);
loadwithprefix(cfg.datafile, 'rn_', comparevars);

% 4. recursive inversion, index optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'recinv';
cfg.indexoptimized = 1;
cfg.datafile = 'test_lspis_recinv_dupd005_indexopt1';
lspionline(cfg);
loadwithprefix(cfg.datafile, 'ro_', comparevars);

for i = 1:30,
    di(i) = norm(in_thetah{i} - io_thetah{i}, 'inf');
    dr(i) = norm(rn_thetah{i} - ro_thetah{i}, 'inf');
    dir(i) = norm(in_thetah{i} - rn_thetah{i}, 'inf');
end;
disp('Diff between opt/nonopt, INV case:');
di
disp('Diff between opt/nonopt, RECINV case:');
dr
disp('Diff between INV and RECINV, nonopt:');
dir
in_trun
io_trun
rn_trun
ro_trun

% LSPI Results: 
%   inv opt/nonopt: slight errors 1e-7 (numerical?)
%   recinv opt/nonopt: 1e-4 errors (numerical?)
%   inv/recinv: 1e-4 errors (expected and acceptable)
% Run time:
%   inv/opt FASTER THAN inv/nonopt FASTER THAN recinv/opt FASTER THAN recinv/nonopt
%   So, best combination is inv/opt. Recinv is hopeless, even index-optimized (10 times
%   slower)
% recinv has its largest advantage in solving the linear system (qsolve)
% However, a study of the computational time for this step shows that while the average time is%
% better, the worst-case time is still the same as for inv -- so no motivation there, either.

% ==============================
% Also try for K_theta = 1 -- if there is no advantage for recinv there, there won't be
% anywhere, so it can safely be removed from the code



%% LSPIonline tests with small update interval ==========================================

lspicfg = ccfg;
lspicfg.dupd = .005;
comparevars = {'thetah', 'trun'};

% Results of 1 should ideally be IDENTICAL with 2

% 1. normal inversion, index optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'inv';
cfg.indexoptimized = 1;
cfg.datafile = 'test_lspis_inv_dupd0005_indexopt1';
% lspionline(cfg);
loadwithprefix(cfg.datafile, 'io_', comparevars);

% 2. recursive inversion, index optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'recinv';
cfg.indexoptimized = 1;
cfg.datafile = 'test_lspis_recinv_dupd0005_indexopt1';
% lspionline(cfg);
loadwithprefix(cfg.datafile, 'ro_', comparevars);

% 3. normal inversion, index not optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'inv';
cfg.indexoptimized = 0;
cfg.datafile = 'test_lspis_inv_dupd0005_indexopt0';
lspionline(cfg);
loadwithprefix(cfg.datafile, 'in_', comparevars);

for i = 1:60,l
    dir(i) = norm(io_thetah{i} - ro_thetah{i}, 'inf');
    di(i) = norm(io_thetah{i} - in_thetah{i}, 'inf');
end;
disp('Diff between INV and RECINV, opt:');
dir
disp('Diff between INV, opt and nonopt:');
di

io_trun
ro_trun
in_trun

% Results: 
%   For dupd = .005
%   There are very large differences between the parameter vectors -- I don't see why...
%   inv/opt STILL FASTER than recinv/opt
% inverse is better even in this case... due to full matrix updates with the
% recursive inverse, the operation ends up costing more than mldivide





%% LSPIonline recinv debug

lspicfg = ccfg;
lspicfg.maxtime = 1;
lspicfg.dupd = .05;

% 2. recursive inversion, index optimized
rand('state', randseed);
cfg = lspicfg;
cfg.method = 'recinv';
cfg.indexoptimized = 1;
cfg.datafile = 'test_lspis_recinv_indexopt1_debug';
cfg.approx = 'enhanced type=rbfdisc N=3 M=3';
lspionline(cfg);
