function [eJ, J] = mc2_fuzzyq(X, U, theta, model, X0, cfg)
% Monte-Carlo evaluation of fuzzy Q-iteration parameter matrix
% In its current form evaluates only deterministic systems

% REMARK terminal state check removed for speed, TODO uncomment when needed

n0 = size(X0, 2);
J = zeros(n0, 1);
% precompute discounting vector, pre-initialize reward vector
disc = cumprod([1 cfg.gamma+zeros(1, cfg.mc_maxsteps)]);
r = zeros(cfg.mc_maxsteps, 1);
p = model.p;
Np = zeros(p, 1);
for i = 1:length(X),
    Np(i) = length(X{i});
end;
roll = 0 * Np;
tab = dec2base(0:2^model.p-1, 2) - 47;       % auxiliary indices table
if cfg.interph,
    [Qstar ui] = max(theta, [], 2);
    hstar = U(:, ui);
end;
for i = 1:n0,
    x = X0(:, i);
    % notice interpolation test taken outside steps loop for speed increase
    if cfg.interph,
        for k = 1:cfg.mc_maxsteps,
            % compute crisp (approximate) optimal action and apply to system
            [ind, mu] = mdegs_p(x, X, roll, Np, p, tab);
            [x r(k) terminal] = feval(model.fun, model, x, hstar(:, ind) * mu);
            % if terminal, break; end;    % no other rewards contribute (equivalently, they are all 0)
        end;
    else
        for k = 1:cfg.mc_maxsteps,
            % compute crisp (approximate) optimal action and apply to system
            [ind, mu] = mdegs_p(x, X, roll, Np, p, tab);
            Qa = mu' * theta(ind, :);
            [x r(k) terminal] = feval(model.fun, model, x, U(:, find(Qa == max(Qa), 1)));
            % if terminal, break; end;    % no other rewards contribute (equivalently, they are all 0)
        end;
    end;
    % return of x0 is inner product of discounting and rewards
    J(i) = disc(1:k) * r(1:k);
end;

% expected value is the mean
eJ = mean(J);

% END rbfmceval() RETURNING expected cost J ===========================

% old, unoptimized version
%         mu = mdegs(x(1), XMFS{1});
%         for p = 2:model.p, mu = mu(:) * mdegs(x(p), XMFS{p})'; end;
%         phi = find(mu); Qa = mu(phi)' * theta(phi, :);
