close all;  clear all;
Uval = 2;
Uvals = '_U2';
%% IP Common config
ccfg = ip_common_config(Uval);
[approx, theta] = fuzzyqi(['approx datafile=ip_fzqi_nearopt' Uvals]);
ccfg.hstar = @(x) approx.h(approx, theta, x);
ccfg.Qstar = @(x, u) approx.q(approx, theta, x, u); 

mfilename = ['ipoff' Uvals]; % Ts=0.025 maxstep=160 gamma=0.98

%% online OPD
cfg = ccfg;
cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 600;         % budget of expansions at each step
cfg.datafile = 'ip_opd';
cfg.saveplandata = 1;
cfg.visualize = 4;             % this means: only visualize system steps
cfg.run = 1;
genmpc(cfg);              

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));


%% online OKP all nodes expanded
cfg = ccfg;
cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 120;         % budget of expansions at each step
cfg.planparams.K = 5;
cfg.planparams.epsil = 10^-10;
cfg.planparams.exploitredundancy = false;
cfg.datafile = 'ip_okp_allnodes';
% cfg.saveplandata = 1;
cfg.visualize = 4;
cfg.run = 1;
genmpc_new(cfg);              

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));


%% online OKP exploiting redundancy
cfg = ccfg;
cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 120;         % budget of expansions at each step
cfg.planparams.K = 5;
cfg.planparams.epsil = 10^-10;
cfg.planparams.exploitredundancy = true;
cfg.datafile = 'ip_okp_expred';
% cfg.saveplandata = 1;
cfg.visualize = 4;             % this means: only visualize system steps
cfg.run = 1;
genmpc(cfg);              

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));


%% online OKP exploiting redundancy equal treesize
cfg = ccfg;
cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
cfg.planparams.nsim = 600;         % budget of expansions at each step
cfg.planparams.K = 5;
cfg.planparams.epsil = 10^-10;
cfg.planparams.exploitredundancy = true;
cfg.datafile = ['ip_okp_expred_eqtree' num2str(cfg.planparams.K)];
% cfg.saveplandata = 1;
cfg.visualize = 4;             % this means: only visualize system steps
cfg.run = 1;
genmpc(cfg);              

figure; hist=genmpc(['plottrial datafile=' cfg.datafile]);
fprintf('total return=%.5f\n\n',hist.R(1));


%% OFFLINE OPD
cfg = ccfg;
cfg.online = 0;
cfg.planalg = 'opd';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 200;         % budget of expansions at each step
cfg.datafile = 'ipoff_opd';
cfg.saveplandata = 1;
cfg.run = 1;
genmpc(cfg);              

%% OFFLINE, eval
% genmpc('ploth datafile=ipoff_opd');
cfg = ccfg;
cfg.datafile = 'ipoff_opd';
opdavgreg = genmpc_exp('regret', cfg)

%% OFFLINE OKP all nodes expanded 
cfg = ccfg;
cfg.online = 0;
cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 50;         % budget of expansions at each step
cfg.planparams.K = 5;
cfg.planparams.epsil = 10^(-10);
cfg.planparams.exploitredundancy = false;
cfg.datafile = 'ipoff_okp_allnodesexpanded';
cfg.saveplandata = 0;
cfg.run = 1;
genmpc(cfg);          


%% OFFLINE, eval
% genmpc('ploth datafile=ipoff_opd');
cfg = ccfg;
cfg.datafile = 'ipoff_okp_allnodesexpanded';
[okpavgreg, okpreg] = genmpc_exp('regret', cfg)

%% OFFLINE OKP exploiting redundancy
cfg = ccfg;
cfg.online = 0;
cfg.planalg = 'okp';            % variant: optimistic planning for deterministic systems
cfg.planparams.n = 192;         % budget of expansions at each step
cfg.planparams.K = 16;
cfg.planparams.epsil = 10^(-10);
cfg.planparams.exploitredundancy = true;
cfg.datafile = 'ipoff_okp_exploitredundancy';
cfg.saveplandata = 1;
cfg.run = 1;
genmpc(cfg);          

% % genmpc('ploth datafile=ipoff_okp_exploitredundancy');

%% OFFLINE, eval
% genmpc('ploth datafile=ipoff_opd');
cfg = ccfg;
cfg.datafile = 'ipoff_okp_exploitredundancy';
[okpplusavgreg, okpplusreg] = genmpc_exp('regret', cfg);

%%
% figure; plot(opdreg, 'bs', 'LineStyle', 'none'); hold on; plot(okpreg, 'g*', 'LineStyle', 'none'); plot(okpplusreg, 'r+', 'LineStyle', 'none'); legend('opd', 'okp', 'okp+');